/**
 *
 */
package cexer;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;

import cexer.util.mceparser.MceConstants;

/**
 * @author cefallen
 *
 */
public class Choice implements Serializable {

    /**
     * Id for serialization.
     */
    private static final long serialVersionUID = 1L;

    /**
     * False if the choice has to be displayed (in its' choice group), true otherwise.
     */
    private boolean isOptional;

    /**
     * Unique question id.
     */
    private int id;

    /**
     * The choice descriptions in different locales.
     */
    private Hashtable descriptions;

    /**
     * Selected feedback for the choice in different locales.
     */
    private Hashtable selectedFeedback;

    /**
     * Unselected feedback for the choice in different locales.
     */
    private Hashtable unselectedFeedback;


    /**
     * The points associated with selecting this choice.
     */
    private int selectedPoints;

    /**
     * The points associated with leaving this choice unselected.
     */
    private int unselectedPoints;

    /**
     * Default constructor.
     */
    public Choice() {
        isOptional = true;
        descriptions = new Hashtable();
        selectedFeedback = new Hashtable();
        unselectedFeedback = new Hashtable();
    }

    /**
     * Constructor, takes the choice id as a parameter.
     *
     * @param id choice id
     */
    public Choice(int id) {
        this.id = id;
        isOptional = true;
        descriptions = new Hashtable();
        selectedFeedback = new Hashtable();
        unselectedFeedback = new Hashtable();
    }

    /**
     * Gets the unique choice id.
     *
     * @return choice id
     */
    public int getId() {
        return id;
    }

    /**
     * Sets the choice id.
     *
     * @param id the choice id
     */
    public void setId(int id) {
        this.id = id;
    }

    /**
     * Gets the status (importance) of this choice within a choice group.
     *
     * @return false if the choice has to be displayed (in its' choice group), true otherwise.
     */
    public boolean isOptional() {
        return isOptional;
    }

    /**
     * Sets the status of this choice. False if the choice has to be displayed (in its' choice group), true otherwise.
     *
     * @param isOptional the boolean state to be set
     */
    public void setOptional(boolean isOptional) {
        this.isOptional = isOptional;
    }

    /**
     * Gets the points associated with selecting this choice.
     *
     * @return number of points
     */
    public int getSelectedPoints() {
        return selectedPoints;
    }

    /**
     * Sets the points associated with selecting this choice.
     *
     * @param selectedPoints the points given
     */
    public void setSelectedPoints(int selectedPoints) {
        this.selectedPoints = selectedPoints;
    }

    /**
     * Gets the points associated with leaving this choice unselected.
     *
     * @return number of points
     */
    public int getUnselectedPoints() {
        return unselectedPoints;
    }

    /**
     * Sets the points associated with leaving this choice unselected.
     *
     * @param unselectedPoints the points given
     */
    public void setUnselectedPoints(int unselectedPoints) {
        this.unselectedPoints = unselectedPoints;
    }

    /**
     * Gets the choice description in the given locale.
     *
     * @param locale the requested locale
     * @return choice description
     */
    public String getDescription(String locale) {
        if(!descriptions.containsKey(locale))
            return (String)descriptions.get(descriptions.keySet().iterator().next());
        else return (String)descriptions.get(locale);
    }

    /**
     * Add a choice description in a new locale.
     *
     * @param description the description text
     * @param lang the locale
     */
    public void addDescription(String description, String lang) {
        descriptions.put(lang, description);
    }

    /**
     * Gets the negative feedback for this choice in the given locale.
     *
     * @param locale the requested locale
     * @return negative feedback text
     */
    public String getUnselectedFeedback(String locale) {
        if(!unselectedFeedback.containsKey(locale))
        	return "";
        else return (String)unselectedFeedback.get(locale);
    }

    /**
     * Add a negative feedback text in a new locale
     *
     * @param feedback the feedback text
     * @param lang the locale
     */
    public void addUnselectedFeedback(String feedback, String lang) {
        unselectedFeedback.put(lang, feedback);
    }

    /**
     * Gets the positive feedback for this choice in the given locale.
     *
     * @param locale the requested locale
     * @return positive feedback text
     */
    public String getSelectedFeedback(String locale) {
        if(!selectedFeedback.containsKey(locale))
        	return "";
        else return (String)selectedFeedback.get(locale);
    }
    
    /**
     * Add a positive feedback text in a new locale
     *
     * @param feedback the feedback text
     * @param lang the locale
     */
    public void addSelectedFeedback(String feedback, String lang) {
        selectedFeedback.put(lang, feedback);
    }

    /**
     * @return the descriptions
     */
    public Hashtable getDescriptions() {
        return descriptions;
    }

    /**
     * @return the negativeFeedback
     */
    public Hashtable getUnselectedFeedbacks() {
        return unselectedFeedback;
    }

    /**
     * @return the positiveFeedback
     */
    public Hashtable getSelectedFeedbacks() {
        return selectedFeedback;
    }

    public void toXml(StringBuffer sb) {
        sb.append("<" + MceConstants.CHOICE_TAG_NAME + " ");
        sb.append(MceConstants.SELECTED_POINTS_ATTRIBUTE_NAME + "=\"" +
                  getSelectedPoints() + "\" ");
        sb.append(MceConstants.UNSELECTED_POINTS_ATTRIBUTE_NAME + "=\"" +
                  getUnselectedPoints() + "\" ");
        sb.append(MceConstants.OPTIONAL_ATTRIBUTE_NAME + "=\"" +
                  isOptional + "\">\n");
        Enumeration i = descriptions.keys();
        while (i.hasMoreElements()) {
            Object next = i.nextElement();
            sb.append("<" + MceConstants.DESCRIPTION_TAG_NAME + " ");
            sb.append(MceConstants.LANG_ATTRIBUTE_NAME + "=\"" +
                      next + "\">");
            sb.append(descriptions.get(next));
            sb.append("</" + MceConstants.DESCRIPTION_TAG_NAME + ">\n");
        }
        i = selectedFeedback.keys();
        while (i.hasMoreElements()) {
            Object next = i.nextElement();
            sb.append("<" + MceConstants.SELECTED_FEEDBACK_TAG_NAME + " ");
            sb.append(MceConstants.LANG_ATTRIBUTE_NAME + "=\"" +
                      next + "\">");
            sb.append(selectedFeedback.get(next));
            sb.append("</" + MceConstants.SELECTED_FEEDBACK_TAG_NAME + ">\n");
        }
        i = unselectedFeedback.keys();
        while (i.hasMoreElements()) {
            Object next = i.nextElement();
            sb.append("<" + MceConstants.UNSELECTED_FEEDBACK_TAG_NAME + " ");
            sb.append(MceConstants.LANG_ATTRIBUTE_NAME + "=\"" +
                      next + "\">");
            sb.append(unselectedFeedback.get(next));
            sb.append("</" + MceConstants.UNSELECTED_FEEDBACK_TAG_NAME + ">\n");
        }
        sb.append("</" + MceConstants.CHOICE_TAG_NAME + ">\n");
    }

}
