/**
 *
 */
package cexer;

import java.io.Serializable;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import cexer.util.mceparser.MceConstants;

/**
 * @author cefallen
 *
 */
public class ChoiceGroup implements Serializable {
    /**
     * Id for serialization.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Choices in this group.
     */
    private List choices;

    /**
     * The number of choices that should be visible to the user when this group is displayed.
     */
    private int numberOfVisibleChoices;

    /**
     * Unique id.
     */
    private int id;

    /**
     * Default constructor.
     */
    public ChoiceGroup() {
        choices = new LinkedList();
        numberOfVisibleChoices = 0;
    }

    /**
     * Constructor, takes the choice group id as a parameter.
     *
     * @param id the choice group id
     */
    public ChoiceGroup(int id) {
        choices = new LinkedList();
        numberOfVisibleChoices = 0;
        this.id = id;
    }

    /**
     * Gets the number of choices that should be visible to the user when this group is displayed.
     *
     * @return number of visible choices.
     */
    public int getNumberOfVisibleChoices() {
        return numberOfVisibleChoices;
    }

    /**
     * Sets the number of choices that should be visible to the user when this group is displayed.
     *
     * @param numberOfVisiblechoices the number of visible choices
     */
    public void setNumberOfVisibleChoices(int numberOfVisiblechoices) {
        this.numberOfVisibleChoices = numberOfVisiblechoices;
    }

    /**
     * Gets all choices in this group.
     *
     * @return list of choices
     */
    public List getChoices() {
        return choices;
    }

    /**
     * Gets a generated choice set. The set contains the number of choices specified in <coded>numberOfVisibleChoices</code>.
     * The set is selected by taking all non-optional choices and the filling up with optionals until the set contains
     * the specified amount. The randomization is done based on the randomSeed and submissionNumber. The same randomseed and
     * submissionNumber gives the same choice set.
     *
     * @param randomSeed the randomseed used for randomizing optional questions
     * @param submissionNumber how many times the exercise containing this choicegroup has been submitted before.
     * @return set of choices
     */
    public List getChoiceSet(long randomSeed, int submissionNumber) {
        List choiceSet = new LinkedList();
        List optionalChoices = new LinkedList();
        Iterator iter = choices.iterator();
        Choice tempChoice = null;

        Random rand = new Random(randomSeed + submissionNumber * submissionNumber);

        while(iter.hasNext()) { //populate the set with mandatory choices
            tempChoice = (Choice)iter.next();
            if(tempChoice.isOptional())
                optionalChoices.add(tempChoice);
            else if(choiceSet.size() < numberOfVisibleChoices)
                choiceSet.add(tempChoice);
        }

        int randomSelectedChoiceNr = 0;
        while(choiceSet.size() < numberOfVisibleChoices && choiceSet.size() < choices.size()) {
            randomSelectedChoiceNr = rand.nextInt(optionalChoices.size());
            tempChoice = (Choice)optionalChoices.get(randomSelectedChoiceNr);
            if(!choiceSet.contains(tempChoice))
                choiceSet.add(tempChoice);
        }
        
        Collections.shuffle(choiceSet, rand);  // randomize choice order
        return choiceSet;
    }

    /**
     * Set the choices in this choice group.
     *
     * @param choices the choices to set
     */
    public void setChoices(List choices) {
        this.choices = choices;
    }

    /**
     * Adds a choice to this choice group.
     *
     * @param choice the choice to add
     */
    public void addChoice(Choice choice) {
        this.choices.add(choice);
    }

    /**
     * Gets the id of this choice group.
     *
     * @return choice group id
     */
    public int getId() {
        return id;
    }

    /**
     * Sets the id of this choice group.
     *
     * @param id the id to set
     */
    public void setId(int id) {
        this.id = id;
    }

    public void toXml(StringBuffer sb) {
        sb.append("<" + MceConstants.CHOICEGROUP_TAG_NAME);
        sb.append(" " + MceConstants.VISIBLE_CHOICES_ATTRIBUTE_NAME +
                  "=\"" + getNumberOfVisibleChoices() + "\">\n");
        for (Iterator iter = getChoices().iterator(); iter.hasNext();) {
            Choice c = (Choice) iter.next();
            c.toXml(sb);
        }
        sb.append("</" + MceConstants.CHOICEGROUP_TAG_NAME + ">");
        //sb.append("</" + MceConstants.MCEXERCISE_TAG_NAME+">\n");
    }
}
