// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.LinkedList;
import java.util.Set;

import java.io.BufferedReader;
import java.io.Serializable;
import java.io.StringReader;

import cexer.util.ExerciseDescriptionParser;

/**
 * This is an exercise, defined by an ExerciseTemplate, which has been
 * attached to a Round. It does not need to know what round it is attached to,
 * or what the actual exercise, defined by the corresponding ExerciseTemplate,
 * is.
 */

public class Exercise implements Serializable {

    private static final long serialVersionUID = 1L;
    
    private int /*points,*/ maxPoints, number, maxSubs;
    private ExerciseTemplate ex;
    // Additional Applet parameters defined here. Must contain only instances of AppletParameters
    private List additionalParameters;
    
    // course specific exercise name in different locales
    private HashMap names;
    // course specific exercise descriptions in different locales
    private HashMap descriptions;
    
    // parsed (and localized) course specific exercise descriptions
    private HashMap localizedTasks;
    private HashMap localizedInstructions;
    private HashMap localizedCodeBlock;
    private HashMap localizedCodeBlockNames;
    private HashMap localizedTabs;
    private HashMap localizedTabNames;

    public Exercise(ExerciseTemplate e, int max, int nro) {
        //	points = 0;
        ex = e;
        if(max >= 0) {
            maxPoints = max;
        } else {
            maxPoints = 1;
        }
        number = nro;
        additionalParameters = null;
        
        names = new HashMap();
        descriptions = new HashMap();
        localizedTasks = new HashMap();
        localizedInstructions = new HashMap();
        localizedCodeBlock = new HashMap();
        localizedCodeBlockNames = new HashMap();
        localizedTabs = new HashMap();
        localizedTabNames = new HashMap();
    }

    public Exercise(ExerciseTemplate e, int max, int nro, List params) {
        this(e,max,nro);
        additionalParameters = params;
    }

    public Exercise(ExerciseTemplate e, int max, int nro, int subs) {
        this(e,max,nro);
        maxSubs = subs;
    }

    public int getNumber() {
        return number;
    }

    public void setNumber(int value) {
        number = value;
    }

    public String getName() { return ex.getName(); }

    public int getMax() {
        return maxPoints;
    }

    public int getSubs() {
        return maxSubs;
    }

    public String getHintPage() {
        if(ex instanceof AppletExerciseTemplate)
            return ((AppletExerciseTemplate)ex).getHintPage();
        return "";
    }

    public String getDescription() { return ex.getDescription(); }

    /**
     * Sets the additional, round-specific applet parameters. To set 
     * exercise-specific parameters, use setAppletParameters() in ExerciseTemplate
     */
    public void setAdditionalAppletParameters(List l) {
        additionalParameters = l;
    }

    public List getAppletParameters() {
        if(ex instanceof AppletExerciseTemplate) {
            List params = ((AppletExerciseTemplate)ex).getAppletParameters();
            // done this way to make it possible to modify exercise parameters in runtime.
            LinkedList ret = new LinkedList(params);
            if(additionalParameters != null)
                ret.addAll(additionalParameters);
            return ret;
        } else {
            return new LinkedList();
        }
    }
    
    /**
     * Returns a list of AppletParameters that contain localized string for the applet ui.
     * If no localized string is found for the specified language, engilsh is used.
     * If no course specific string is found, global default is used.  
     * @param lang Two letter language id. Null falls back to english
     * @param course. Can be null.
     * @return Empty list if nothing was found
     */
    public List getLocalizedStrings(String lang, Course course) {
        if (lang == null) {
            lang = "en";
        }
        
        int cid = -1;
        if (course != null) {
            cid = course.getId();
        }
        
        DataStorage ds = DataStorage.getInstance();
        
        // We don't want to load this data on object creation, because most of the time we don't need it.
        return ds.getLocalizedAppletStrings(ex.getId(), lang, cid);
    }


    public ExerciseTemplate getExerciseTemplate() {
        return ex;
    }

    public void addUse() {
        ex.addUse();
    }

    public void removeUse() {
        ex.removeUse();
    }

    public void getUses() {
        ex.getUses();
    }

    public List getSupportedLocales() {
        if(names.keySet().size() > 0) {
            Set set = new LinkedHashSet();
            set.addAll(names.keySet());
            set.addAll(ex.getSupportedLocales());
            return new LinkedList(set);
        } else { 
            return ex.getSupportedLocales();
        }
    }

    public String getLocalizedName(String lang) {
        if(names.containsKey(lang)) {
            return (String)names.get(lang);
        } else {
            return ex.getLocalizedName(lang);
        }
    }

    public String getLocalizedDescription(String lang) {
        if(descriptions.containsKey(lang)) {
            return (String)descriptions.get(lang);
        } else  {
            return ex.getLocalizedDescription(lang);
        }
    }
    
    public String getInstructions() {
        // XXX: Use current locale
        return getLocalizedInstructions("en");

        // return ex.getInstructions();
    }
    
    public String getLocalizedInstructions(String lang) {
        if(localizedInstructions.containsKey(lang)) {
            return (String)localizedInstructions.get(lang);
        } else {
            return ex.getLocalizedInstructions(lang);
        }
    }    
    
    public String getTask() {
        return ex.getTask();
    }
    
    public String getLocalizedTask(String lang) {
        if(localizedTasks.containsKey(lang)) {
            return (String)localizedTasks.get(lang);
        } else {
            return ex.getLocalizedTask(lang);
        }
    }
    
    public LinkedList getCodeBlockNames() {
        return ex.getCodeBlockNames();
    }
    
    public LinkedList getLocalizedCodeBlockNames(String lang) {
        if(localizedCodeBlockNames.containsKey(lang)) {
            return (LinkedList)localizedCodeBlockNames.get(lang);
        } else {
            return ex.getLocalizedCodeBlockNames(lang);
        }
    }
    
    public String getCodeBlock(String blockName) {
        return ex.getCodeBlock(blockName);
    }
    
    public String getLocalizedCodeBlock(String blockName, String lang) {
        if (localizedCodeBlock.containsKey(lang)) {
            return (String)((HashMap)localizedCodeBlock.get(lang)).get(blockName);
        } else {
            return ex.getLocalizedCodeBlock(blockName, lang);
        }
    }
    
    public List getTabNames() {
        return ex.getTabNames();
    }
    
    public List getLocalizedTabNames(String lang) {
        if(localizedTabNames.containsKey(lang)) {
            return (LinkedList)localizedTabNames.get(lang);
        } else {
            return ex.getLocalizedTabNames(lang);
        }
    }
    
    public String getTab(String tabName) {
        return ex.getTab(tabName);
    }
    
    public String getLocalizedTab(String tabName, String lang) {
        if (localizedTabs.containsKey(lang)) {
            return (String)((HashMap)localizedTabs.get(lang)).get(tabName);
        } else {
            return ex.getLocalizedTab(tabName, lang);
        }
    }
    
    /**
     * Tells if the exercise has a course-specific description in the given language
     * 
     * @param lang language identifier
     * @return true if the exercise has a course-specific description in the given language
     */
    public boolean hasCourseSpecificDescription(String lang) {
        return names.containsKey(lang);
    }
    
    public void addLocalizedData(String language, String name, String description) {
        names.put(language, name);
        descriptions.put(language, description);

        ExerciseDescriptionParser parser = new ExerciseDescriptionParser(new BufferedReader(new StringReader(description)));
        localizedInstructions.put(language, parser.getInstructions());
        localizedTasks.put(language, parser.getTask());
        
        if(!parser.getCodeBlocks().isEmpty()) {
            localizedCodeBlock.put(language, parser.getCodeBlocks());
        }
        
        if(parser.getCodeBlockNames().size() > 0) {
            localizedCodeBlockNames.put(language, parser.getCodeBlockNames());
        }
        
        if(!parser.getTabs().isEmpty()) {
            localizedTabs.put(language, parser.getTabs());
        }
        
        if(parser.getTabNames().size() > 0) {
            localizedTabNames.put(language, parser.getTabNames());
        }
    }

    public void removeLocalizedData(String language) {
        names.remove(language);
        descriptions.remove(language);
        localizedInstructions.remove(language);
        localizedTasks.remove(language);
        localizedCodeBlock.remove(language);
        localizedCodeBlockNames.remove(language);
        localizedTabs.remove(language);
        localizedTabNames.remove(language);
    }
}
