// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.HashMap;
import java.util.List;
import java.util.LinkedList;
import java.io.Serializable;
import java.io.BufferedReader;
import java.io.StringReader;
import cexer.util.ExerciseDescriptionParser;

/**
 * This is a template of an exercise that includes all basic information about the
 * exercise.
 */
public class ExerciseTemplate implements Serializable {
    // name of exercise to be shown on the web page
    private String name;
    private HashMap names;
    // exercise description XXX localization!
    private String description;
    private HashMap descriptions;
    
    // parsed (and localized) exercise descriptions:
    private String task;
    private HashMap localizedTasks;
    private String instructions;
    private HashMap localizedInstructions;
    private HashMap codeBlocks;
    private LinkedList codeBlockNames;
    private HashMap tabs;
    private List tabNames;
    private HashMap localizedCodeBlock;
    private HashMap localizedCodeBlockNames;
    private HashMap localizedTabs;
    private HashMap localizedTabNames;
    


    private int id;

    private int uses;

    public ExerciseTemplate(String name, String description, List p) {
        this.name = name;
        this.description = description;

        uses = 0;
        descriptions = new HashMap();
        names = new HashMap();

        //Universal language description parser
        ExerciseDescriptionParser parser = new ExerciseDescriptionParser(new BufferedReader(new StringReader(description)));
        task = parser.getTask();
        codeBlocks = parser.getCodeBlocks();
        codeBlockNames = parser.getCodeBlockNames();
        instructions = parser.getInstructions();
        tabs = parser.getTabs();
        tabNames = parser.getTabNames();

        //HashMaps for localized versions
        localizedInstructions = new HashMap();
        localizedTasks = new HashMap();
        localizedCodeBlock = new HashMap();
        localizedCodeBlockNames = new HashMap();
        localizedTabs = new HashMap();
        localizedTabNames = new HashMap();

    }

    public String getName() {
        return name;
    }

    public void setName(String value) {
        name = value;
    }

    public void setDescription(String value) {
        description = value;
    }

    public String getDescription() {
        return description;
    }

    public String getTask() {
        return task;
    }

    public String getInstructions() {
        return instructions;
    }

    public String getCodeBlock(String name) {
        return (String)codeBlocks.get(name);
    }

    public LinkedList getCodeBlockNames() {
        return codeBlockNames;
    }

    public String getTab(String name) {
        return (String)tabs.get(name);
    }

    public List getTabNames() {
        return tabNames;
    }

    public void addUse() {
    uses++;
    }

    public void removeUse() {
        uses--;
    }

    public int getUses() {
        return uses;
    }

    public List getSupportedLocales() {
        return new LinkedList(names.keySet());
    }

    public void addLocalizedData(String language, String name, String description) {
        names.put(language, name);
        descriptions.put(language, description);

        ExerciseDescriptionParser parser = new ExerciseDescriptionParser(new BufferedReader(new StringReader(description)));
        localizedInstructions.put(language, parser.getInstructions());
        localizedTasks.put(language, parser.getTask());

        if (!parser.getCodeBlocks().isEmpty()) {
            localizedCodeBlock.put(language, parser.getCodeBlocks());
        }

        if (parser.getCodeBlockNames().size() > 0) {
            localizedCodeBlockNames.put(language, parser.getCodeBlockNames());
        }
        
        if(!parser.getTabs().isEmpty()) {
            localizedTabs.put(language, parser.getTabs());
        }
        
        if(parser.getTabNames().size() > 0) {
            localizedTabNames.put(language, parser.getTabNames());
        }
    }

    public String getLocalizedName(String language) {
        if (names.containsKey(language)) {
            return (String)names.get(language);
        } else {
            return getName();
        }
    }

    public String getLocalizedDescription(String language) {
        if (descriptions.containsKey(language)) {
            return (String)descriptions.get(language);
        } else {
            return getDescription();
        }
    }

    public String getLocalizedInstructions(String language) {
        if (localizedInstructions.containsKey(language)) {
            return (String)localizedInstructions.get(language);
        } else {
            return getInstructions();
        }
    }

    public String getLocalizedTask(String language) {
        if (localizedTasks.containsKey(language)) {
            return (String)localizedTasks.get(language);
        } else {
            return getTask();
        }
    }

    public String getLocalizedCodeBlock(String blockName, String language) {
        if (localizedCodeBlock.containsKey(language)) {
            return (String)((HashMap)localizedCodeBlock.get(language)).get(blockName);
        } else {
            return getCodeBlock(blockName);
        }
    }

    public LinkedList getLocalizedCodeBlockNames(String language) {
        if (localizedCodeBlockNames.containsKey(language)) {
            return (LinkedList)localizedCodeBlockNames.get(language);
        } else {
            return getCodeBlockNames();
        }
    }

    public String getLocalizedTab(String tabName, String language) {
        if (localizedTabs.containsKey(language)) {
            return (String)((HashMap)localizedTabs.get(language)).get(tabName);
        } else {
            return getTab(tabName);
        }
    }

    public List getLocalizedTabNames(String language) {
        if (localizedTabNames.containsKey(language)) {
            return (List)localizedTabNames.get(language);
        } else {
            return getTabNames();
        }
    }

    public void setId(int value) {
        id = value;
    }

    public int getId() {
        return id;
    }
}
