/**
 * 
 */
package cexer;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Answer state object for a multiple choice exercise. Used for saving user selections and grading.
 * 
 * @author cefallen
 *
 */
public class MultipleChoiceExerciseAnswer {
    
    /**
     * Contains all questions sets for this  multiple choice exercise.
     */
    private List questionSets;
    
    /**
     * Maps question id to a list containing the numbers of the selected choices.
     */
    private Hashtable selectedChoices;
    
    /**
     * Maps question id to a list of all generated choice sets associated with this answer instance 
     * of the question.
     */
    private Hashtable choiceSets;
    
    /**
     * The random seed used to generate this exercise instance.
     */
    private long randomSeed;
    
    /**
     * Number of submissions made before this answer was generated.
     */
    private int submissions;
    
    /**
     * Tells if this answer has been submitted.
     */
    private boolean submitted;
    
    /**
     * The exercise id of the exercise which this answer belongs to.
     */
    private int exerciseId;

    /**
     * Constructor. Creates a answer for the provided list of questionGroups.
     * 
     * @param questions list of questiongroups that this answer should be created for
     * @param randomSeed random seed used for creating the question and choice sets
     * @param submissions number of submissions made, used for modifying the random seed to create a new
     * question instance for each submission
     */
    public MultipleChoiceExerciseAnswer(List questions, long randomSeed, int submissions) {
    	
        this.randomSeed = randomSeed;
        this.submissions = submissions;
        questionSets = new LinkedList();
        selectedChoices = new Hashtable();
        choiceSets = new Hashtable();
        
        this.submitted = false;
        
        QuestionGroup tempGroup = null;
        ChoiceGroup tempChoiceGroup = null;
        Question tempQuestion = null;
        List tempSet = null;
        List tempChoiceSet = null;
        
        Iterator iter = questions.iterator();
        Iterator questionIterator = null;
        Iterator choiceGroupIterator = null;
        
        while(iter.hasNext()) {
            tempGroup = (QuestionGroup)iter.next();
            tempSet = tempGroup.getQuestionSet(randomSeed, submissions);
            questionSets.add(tempSet);
            questionIterator = tempSet.iterator();
            while(questionIterator.hasNext()) {
                tempChoiceSet = new LinkedList();
                tempQuestion = (Question)questionIterator.next();                
                choiceGroupIterator = tempQuestion.getChoiceGroups().iterator();
                tempQuestion.getChoiceSet(randomSeed, submissions); // update the choiceset for each question
                while(choiceGroupIterator.hasNext()) {
                    tempChoiceGroup = (ChoiceGroup)choiceGroupIterator.next();
                    tempChoiceSet.add(tempChoiceGroup.getChoiceSet(randomSeed, submissions));                    
                }
                choiceSets.put(new Integer(tempQuestion.getId()), tempChoiceSet);
                selectedChoices.put(new Integer(tempQuestion.getId()), new LinkedList());
            }
        }        
    }

    /**
     * Gets the number of the selected radio choice for the given question id
     * 
     * @param questionId id of the question
     * @return number of the selected choice
     */
    public int getSelectedRadioChoice(int questionId) {        
        List tempList = (List)selectedChoices.get(new Integer(questionId));
        int value = 0;
        if(tempList != null && tempList.size() > 0)
            value = Integer.parseInt((String)tempList.get(0));
        return value;
    }
    
    /**
     * Sets the selected radio button choice number for a question
     * 
     * @param questionId the question id
     * @param choice number of the selected choice
     */
    public void setSelectedRadioChoice(int questionId, int choice) {
        List tempList = new LinkedList();
        tempList.add(Integer.toString(choice));
        selectedChoices.put(new Integer(questionId), tempList);
    }
    
    /**
     * Gets the choice with the given number of the provided question.
     * 
     * @param question the question
     * @param choiceNumber the number of the choice to get
     * @return choice
     */
    private Choice getChoiceByNumber(Question question, int choiceNumber) {
    	return (Choice)question.getSavedChoiceSet().get(choiceNumber);
   }     
    
    /**
     * Gets a list containing all choices for the current instance of the provided question
     * 
     * @param question the question
     * @return list of choices
     */
    private List getChoicesForQuestion(Question question) {
        List choiceSet = (List)choiceSets.get(new Integer(question.getId()));
        Iterator choiceSetIter = choiceSet.iterator();
        Iterator choiceIter = null;
        List choices = new LinkedList();
        
        while(choiceSetIter.hasNext()) {
            choiceIter = ((List)choiceSetIter.next()).iterator();
            while(choiceIter.hasNext()) {
                choices.add((Choice)choiceIter.next());
            }
        }
        return choices; // list is in the original order (same as in monitor view)
    }

    /**
     * Calculates the points for this answer.
     * 
     * @return points
     */
    public int getPoints() {
        Iterator questionSetIterator = questionSets.iterator();
        Iterator questionIterator = null;
        Question tempQuestion = null;        
        int points = 0;
        
        while(questionSetIterator.hasNext()) {
            questionIterator = ((List)questionSetIterator.next()).iterator();
            while(questionIterator.hasNext()) {
                tempQuestion = (Question)questionIterator.next();
                points = points + getQuestionPoints(tempQuestion);                   
            }
        }
        // Don't allow negative points for an exercise. Negative points
        // are however allowed for single questions. -EF
        if(points < 0)
            points = 0;
        return points;
    }
    
    /**
     * Calculates the points for a single question.
     * 
     * @param question the question
     * @return points for the question
     */
    public int getQuestionPoints(Question question) {
        Choice tempChoice = null;
        int points = 0;
        List choiceList = question.getSavedChoiceSet();
        Iterator selectedIterator = ((List)selectedChoices.get(new Integer(question.getId()))).iterator(); // get the numbers of the selected options
        
        while(selectedIterator.hasNext()) {
            tempChoice = getChoiceByNumber(question, Integer.parseInt((String) selectedIterator.next()));
            if(tempChoice != null) {
                points = points + tempChoice.getSelectedPoints(); // add selected points           
                choiceList.remove(tempChoice); // remove from unselected list
            }
        }
        selectedIterator = choiceList.iterator();
        while(selectedIterator.hasNext()) {
            tempChoice = (Choice)selectedIterator.next();
            points = points + tempChoice.getUnselectedPoints(); // add unselected points
        }
        
        return points;
    }

    /**
     * Sets the selected value for a choice in the given question.
     * 
     * @param questionId the question
     * @param choiceNumber number of the choice to set or unset
     * @param selected the new value of the choice, true if selected
     */
    public void setCheckboxChoiceSelected(int questionId, int choiceNumber, boolean selected) {
    	
        List tempList = (List)selectedChoices.get(new Integer(questionId));
        if(tempList == null)
            tempList = new LinkedList();
        if(selected && !tempList.contains(Integer.toString(choiceNumber)))
            tempList.add(Integer.toString(choiceNumber));
        if(!selected)
            tempList.remove(Integer.toString(choiceNumber));
        selectedChoices.put(new Integer(questionId), tempList);       
    }
    
    /**
     * Gets the selected value for a choice in the given question
     * 
     * @param questionId the question
     * @param choiceNumber number of the choice
     * @return true if selected, false otherwise
     */
    public boolean getCheckboxChoiceSelected(int questionId, int choiceNumber) {
        List tempList = (List)selectedChoices.get(new Integer(questionId));
        if(tempList != null && tempList.contains(Integer.toString(choiceNumber)))
            return true;
        else return false;
    }

    /**
     * Gets the random seed used to generate this exercise instance.
     * 
     * @return random seed
     */
    public long getRandomSeed() {
        return randomSeed;
    }

    /**
     * Gets the number of submissions made before this answer was generated.
     * 
     * @return number of submissions made
     */
    public int getSubmissions() {
        return submissions;
    }
        
    /**
     * Tells if the answer has been submitted.
     * 
     * @return true if submitted, false otherwise
     */
    public boolean isSubmitted() {
        return submitted;
    }

    /**
     * Sets the value of the <code>submitted</code> field.
     * 
     * @param submitted true if submitted, false otherwise
     */
    public void setSubmitted(boolean submitted) {
        this.submitted = submitted;
    }

    /**
     * Gets the maximum possible points for this questionaire/answer instance
     * 
     * @return maximum points
     */
    public int getMaxPoints() {
        Iterator questionSetIterator = questionSets.iterator();
        Iterator questionIterator = null;
        Question tempQuestion = null;        
        int maxPoints = 0;
        
        while(questionSetIterator.hasNext()) {
            questionIterator = ((List)questionSetIterator.next()).iterator();
            while(questionIterator.hasNext()) {
                tempQuestion = (Question)questionIterator.next();
                maxPoints = maxPoints + tempQuestion.getMaxPoints(randomSeed, submissions);                   
            }
        }        
        return maxPoints;
    }

    /**
     * Gets the exercise id of the exercise which this answer belongs to.
     * 
     * @return exercise id
     */
    public int getExerciseId() {
        return exerciseId;
    }

    /**
     * Sets the exercise id of the exercise that this answer belongs to.
     * 
     * @param exerciseId
     */
    public void setExerciseId(int exerciseId) {
        this.exerciseId = exerciseId;
    }
    
    /**
     * Gets the amount of selected choices of a spesific question
     * @param question
     * @return amount of selected choices
     */
    public int getSelectedCount(Question question){
    	List tempList = (List)selectedChoices.get(new Integer(question.getId()));
    	if (tempList != null){
    		return tempList.size();
    	}
    	return 0;
    }
}
