/**
 *
 */
package cexer;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import cexer.util.mceparser.MceConstants;

/**
 *
 * @author cefallen
 *
 */
public class Question implements Serializable {

    /**
     * Id for serialization.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Choicegroups associated with this question.
     */
    private List choiceGroups;

    /**
     * False if the question has to be displayed (in its' question group), true otherwise.
     */
    private boolean isOptional;

    /**
     * Unique question id.
     */
    private int id;

    /**
     * Minimum number of choices that the user has to select. If both <code>maxSelected</code>
     * and <code>minSelectes</code> are 1, the choices are shown as a radio button group.
     */
    private int minSelected;

    /**
     * Maximum number of choices that the user is allowed to select. If both <code>maxSelected</code>
     * and <code>minSelectes</code> are 1, the choices are shown as a radio button group.
     */
    private int maxSelected;

    /**
     * The question descriptions in different locales.
     */
    private Hashtable descriptions;

    /**
     * Positive feedback for the question in different locales.
     */
    private Hashtable positiveFeedback;

    /**
     * Negative feedback for the question in different locales.
     */
    private Hashtable negativeFeedback;
    
    /**
     * True if the choice descriptions of the question are preformatted text
     */
    private boolean preformatted;
    
    
    /**
     * True if choices are layed out in a single row.
     * False if choices are layed out in separate rows.
     */
    private boolean columnLayout;
    
    private List choiceSet = null;
    private String infoText = "";

    /**
     * Default constructor.
     */
    public Question() {
        choiceGroups = new LinkedList();
        isOptional = true;
        descriptions = new Hashtable();
        positiveFeedback = new Hashtable();
        negativeFeedback = new Hashtable();
        minSelected = maxSelected = 0;
    }

    /**
     * Constructor, takes the question id as a parameter.
     *
     * @param id question id
     */
    public Question(int id) {
        choiceGroups = new LinkedList();
        isOptional = true;
        this.id = id;
        descriptions = new Hashtable();
        positiveFeedback = new Hashtable();
        negativeFeedback = new Hashtable();
        minSelected = maxSelected = 0;
    }

    /**
     * Gets all choice groups associated with this question.
     *
     * @return list of choice groups
     */
    public List getChoiceGroups() {
        return choiceGroups;
    }

    /**
     * Set the choice groups of this question.
     *
     * @param choiceGroups list of choicegroups to set
     */
    public void setChoiceGroups(List choiceGroups) {
        this.choiceGroups = choiceGroups;
    }

    /**
     * Adds a choice group to the question.
     *
     * @param choiceGroup the choice group to add
     */
    public void addChoiceGroup(ChoiceGroup choiceGroup) {
        this.choiceGroups.add(choiceGroup);
    }

    /**
     * Gets the choices for a question in a randomized order,
     * without grouping them into choicegroups.
     * 
     * @param randomSeed random seed to use
     * @param submissions number of used submissions 
     * @return choices in randomized order
     */
    public List getChoiceSet(long randomSeed, int submissions) {
        choiceSet = new ArrayList();
        Iterator choiceGroupIterator = getChoiceGroups().iterator();
        Iterator choiceIterator = null;
        ChoiceGroup tempChoiceGroup = null;
        
        while(choiceGroupIterator.hasNext()) {
            tempChoiceGroup = (ChoiceGroup)choiceGroupIterator.next();
            choiceIterator = tempChoiceGroup.getChoiceSet(randomSeed, submissions).iterator();
            while(choiceIterator.hasNext())
                choiceSet.add(choiceIterator.next());
        }
        Random rand = new Random(randomSeed + getId());
        Collections.shuffle(choiceSet, rand);
        return choiceSet;
    }
    
    /**
     * Get the question's saved choice set
     * @return the saved choice set
     */
    public List getSavedChoiceSet(){
    	List copySet = new ArrayList();
    	for (int i=0; i < choiceSet.size(); i++){
    		copySet.add(choiceSet.get(i));
    	}
    	return copySet;
    }
    
    
    /**
     * Gets the status (importance) of this question within a question group.
     *
     * @return false if the question has to be displayed (in its' question group), true otherwise.
     */
    public boolean isOptional() {
        return isOptional;
    }

    /**
     * Sets the status of this question. False if the question has to be displayed (in its' question group), true otherwise.
     *
     * @param isOptional the boolean state to be set
     */
    public void setOptional(boolean isOptional) {
        this.isOptional = isOptional;
    }

    /**
     * Gets the unique question id.
     *
     * @return
     */
    public int getId() {
        return id;
    }

    /**
     * Sets the question id.
     *
     * @param id the question to set
     */
    public void setId(int id) {
        this.id = id;
    }

    /**
     * Gets the question description in the given locale.
     *
     * @param locale the requested locale
     * @return question description
     */
    public String getDescription(String locale) {
        if(!descriptions.containsKey(locale))
            return (String)descriptions.get(descriptions.keySet().iterator().next());
        else return (String)descriptions.get(locale);
    }

    /**
     * Add a question description in a new locale.
     *
     * @param description the description text
     * @param lang the locale
     */
    public void addDescription(String description, String lang) {
        descriptions.put(lang, description);
    }

    /**
     * Gets the negative feedback for this question in the given locale.
     *
     * @param locale the requested locale
     * @return negative feedback text
     */
    public String getNegativeFeedback(String locale) {
        if(!negativeFeedback.containsKey(locale))
        	return "";
        else return (String)negativeFeedback.get(locale);
    }

    /**
     * Add a negative feedback text in a new locale
     *
     * @param feedback the feedback text
     * @param lang the locale
     */
    public void addNegativeFeedback(String feedback, String lang) {
        negativeFeedback.put(lang, feedback);
    }

    /**
     * Gets the positive feedback for this question in the given locale.
     *
     * @param locale the requested locale
     * @return positive feedback text or empty string if feedback is not found
     */
    public String getPositiveFeedback(String locale) {
        if(!positiveFeedback.containsKey(locale)){
        	return "";
        }
        else return (String)positiveFeedback.get(locale);
    }

    /**
     * Add a positive feedback text in a new locale
     *
     * @param feedback the feedback text
     * @param lang the locale
     */
    public void addPositiveFeedback(String feedback, String lang) {
        positiveFeedback.put(lang, feedback);
    }

    /**
     * Gets the maximum number of choices that the user is allowed to select.
     *
     * @return number of choices
     */
    public int getMaxSelected() {
        return maxSelected;
    }

    /**
     * Sets the maximum number of choices that the user is allowed to select.
     * If both <code>maxSelected</code> and <code>minSelectes</code> are 1,
     * the choices are shown as a radio button group.
     *
     * @param maxSelected the maximum number of selected choices
     */
    public void setMaxSelected(int maxSelected) {
        this.maxSelected = maxSelected;
    }

    /**
     * Gets the minimum number of choices that the user has to select.
     *
     * @return number of choices
     */
    public int getMinSelected() {
        return minSelected;
    }

    /**
     * Sets the the minimum number of choices that the user has to select.
     * If both <code>maxSelected</code> and <code>minSelectes</code> are 1,
     * the choices are shown as a radio button group.
     *
     * @param maxSelected the maximum number of selected choices
     */
    public void setMinSelected(int minSelected) {
        this.minSelected = minSelected;
    }

    /**
     * Gets the maximum points for a certain instance of this question. The
     * instance is defined by the parameters randomseed and submissions.
     *
     * @param randomSeed the randomseed used for generating the question content
     * @param submissions submission number
     * @return
     */
    public int getMaxPoints(long randomSeed, int submissions) {
        ChoiceGroup tempChoiceGroup = null;
        Choice tempChoice = null;
        List choiceSet = new LinkedList();
        int points = 0;
        int highSelected = 0;
        int highUnselected = 0;
        boolean isRadioButtonQuestion = false;

        if(minSelected == 1 && maxSelected == 1)
            isRadioButtonQuestion = true;

        Iterator choiceGroupIterator = getChoiceGroups().iterator();
        while(choiceGroupIterator.hasNext()) {
            tempChoiceGroup = (ChoiceGroup)choiceGroupIterator.next();
            choiceSet.add(tempChoiceGroup.getChoiceSet(randomSeed, submissions));
        }

        Iterator choiceSetIter = choiceSet.iterator();
        Iterator choiceIter = null;

        while(choiceSetIter.hasNext()) {
            choiceIter = ((List)choiceSetIter.next()).iterator();
            while(choiceIter.hasNext()) {
                tempChoice = ((Choice)choiceIter.next());
                if(isRadioButtonQuestion) {
                    if(tempChoice.getSelectedPoints() > tempChoice.getUnselectedPoints() && tempChoice.getSelectedPoints() > highSelected) {
                        points = points + tempChoice.getSelectedPoints();
                        points = points - highSelected;
                        points = points + highUnselected;
                        highSelected = tempChoice.getSelectedPoints();
                        highUnselected = tempChoice.getUnselectedPoints();
                    } else {
                        points = points + tempChoice.getUnselectedPoints();
                    }

                } else
                if(tempChoice.getSelectedPoints() > tempChoice.getUnselectedPoints())
                    points = points + tempChoice.getSelectedPoints();
                else points = points + tempChoice.getUnselectedPoints();
            }
        }
        return points;
    }

    public Hashtable getDescriptions() {
        return descriptions;
    }

    public Hashtable getNegativeFeedbacks() {
        return negativeFeedback;
    }

    public Hashtable getPositiveFeedbacks() {
        return positiveFeedback;
    }

    public void toXml(StringBuffer sb) {
        sb.append("<" + MceConstants.QUESTION_TAG_NAME + " ");
        sb.append(MceConstants.MIN_SELECTED_ATTRIBUTE_NAME + "=\"" +
                  getMinSelected() + "\" ");
        sb.append(MceConstants.MAX_SELECTED_ATTRIBUTE_NAME + "=\"" +
                  getMaxSelected() + "\" ");
        sb.append(MceConstants.OPTIONAL_ATTRIBUTE_NAME + "=\"" +
                  isOptional + "\" ");
        sb.append(MceConstants.PREFORMATTED + "=\"" +
                preformatted + "\" ");
        sb.append(MceConstants.CHOICES_LAYOUT + "=\"" +
                columnLayout + "\">\n");
        Enumeration i = descriptions.keys();
        while (i.hasMoreElements()) {
            Object next = i.nextElement();
            sb.append("<" + MceConstants.DESCRIPTION_TAG_NAME + " ");
            sb.append(MceConstants.LANG_ATTRIBUTE_NAME + "=\"" +
                      next + "\">");
            sb.append(descriptions.get(next));
            sb.append("</" + MceConstants.DESCRIPTION_TAG_NAME + ">\n");
        }
        i = positiveFeedback.keys();
        while (i.hasMoreElements()) {
            Object next = i.nextElement();
            sb.append("<" + MceConstants.POSITIVE_FEEDBACK_TAG_NAME + " ");
            sb.append(MceConstants.LANG_ATTRIBUTE_NAME + "=\"" +
                      next + "\">");
            sb.append(positiveFeedback.get(next));
            sb.append("</" + MceConstants.POSITIVE_FEEDBACK_TAG_NAME + ">\n");
        }
        i = negativeFeedback.keys();
        while (i.hasMoreElements()) {
            Object next = i.nextElement();
            sb.append("<" + MceConstants.NEGATIVE_FEEDBACK_TAG_NAME + " ");
            sb.append(MceConstants.LANG_ATTRIBUTE_NAME + "=\"" +
                      next + "\">");
            sb.append(negativeFeedback.get(next));
            sb.append("</" + MceConstants.NEGATIVE_FEEDBACK_TAG_NAME + ">\n");
        }
        for (Iterator iter = getChoiceGroups().iterator(); iter.hasNext();) {
            ChoiceGroup cg = (ChoiceGroup) iter.next();
            cg.toXml(sb);
        }


        sb.append("</" + MceConstants.QUESTION_TAG_NAME + ">\n");
    }

    /**
     * Set a info text for the question
     * @param s the info text
     */
    public void setInfo(String s){
    	infoText = s;
    }
    
    /**
     * Get the info text of the question
     * @return the info text
     */
    public String getInfo(){
    	return infoText;
    }
    
    /**
     * Set whether the choice text should be preformatted or not
     * @param p
     */
    public void setPreformatted(boolean p){
    	preformatted = p;
    }
    
    /**
     * @return True if choice texts should be preformatted
     */
    public boolean isPreformatted(){
    	return preformatted;
    }
    
    /**
     * Set whether the choices should be layed out in separate columns
     */
    public void setColumnLayout(boolean cl){
    	columnLayout= cl;
    }
    
    /**
     * 
     * @return 	True if choices are layed out in a single row.
     * 			False if choices are layed out in separate rows.
     */
    public boolean isColumnLayout(){
    	return columnLayout;
    }
}
