/**
 *
 */
package cexer;

import java.io.Serializable;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import cexer.util.mceparser.MceConstants;

/**
 * @author cefallen
 *
 */
public class QuestionGroup implements Serializable {

    /**
     * Id for serialization.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Questions in this group.
     */
    private List questions;

    /**
     * The number of questions that should be visible to the user when this group is displayed.
     */
    private int numberOfVisibleQuestions;

    /**
     * Unique id.
     */
    private int id;

    /**
     * Default constructor.
     */
    public QuestionGroup() {
        questions = new LinkedList();
        numberOfVisibleQuestions = 0;
    }

    /**
     * Constructor, takes the question group id as a parameter.
     *
     * @param id the question group id
     */
    public QuestionGroup(int id) {
        questions = new LinkedList();
        numberOfVisibleQuestions = 0;
        this.id = id;
    }

    /**
     * Adds a question to this questuon group.
     *
     * @param question the question to add
     */
    public boolean addQuestion(Question question) {
        return questions.add(question);
    }

    /**
     * Gets the number of questions that should be visible to the user when this group is displayed.
     *
     * @return number of visible questions.
     */
    public int getNumberOfVisibleQuestions() {
        return numberOfVisibleQuestions;
    }

    /**
     * Sets the number of questions that should be visible to the user when this group is displayed.
     *
     * @param numberOfVisibleQuestions the number of visible questions
     */
    public void setNumberOfVisibleQuestions(int numberOfVisibleQuestions) {
        this.numberOfVisibleQuestions = numberOfVisibleQuestions;
    }

    /**
     * Gets all questions in this group.
     *
     * @return list of questions
     */
    public List getQuestions() {
        return questions;
    }

    /**
     * Gets a generated question set. The set contains the number of questions specified in <coded>numberOfVisibleQuestions</code>.
     * The set is selected by taking all non-optional questions and the filling up with optionals until the set contains
     * the specified amount. The randomization is done based on the randomSeed and submissionNumber. The same randomseed and
     * submissionNumber gives the same question set.
     *
     * @param randomSeed the randomseed used for randomizing optional questions
     * @param submissionNumber how many times the exercise containing this questiongroup has been submitted before.
     * @return set of questions
     */
    public List getQuestionSet(long randomSeed, int submissionNumber) {
        List questionSet = new LinkedList();
        List optionalQuestions = new LinkedList();
        Iterator iter = questions.iterator();
        Question tempQuestion = null;

        Random rand = new Random(new Random(randomSeed + submissionNumber * id).nextInt());

        while(iter.hasNext()) { //populate the set with mandatory questions
            tempQuestion = (Question)iter.next();
            if(tempQuestion.isOptional())
                optionalQuestions.add(tempQuestion);
            else if(questionSet.size() < numberOfVisibleQuestions)
                questionSet.add(tempQuestion);
        }

        int randomSelectedQuestionNr = 0;
        while(questionSet.size() < numberOfVisibleQuestions && questionSet.size() < questions.size()) {
            randomSelectedQuestionNr = rand.nextInt(optionalQuestions.size());
            tempQuestion = (Question)optionalQuestions.get(randomSelectedQuestionNr);
            if(!questionSet.contains(tempQuestion))
                questionSet.add(tempQuestion);
        }

        Collections.shuffle(questionSet, rand); // randomize the order of the questions
        return questionSet;
    }

    /**
     * Set the questions in this question group.
     *
     * @param questions the questions to set
     */
    public void setQuestions(List questions) {
        this.questions = questions;
    }

    /**
     * Gets the id of this question group.
     *
     * @return question group id
     */
    public int getId() {
        return id;
    }

    /**
     * Sets the id of this question group.
     *
     * @param id the id to set
     */
    public void setId(int id) {
        this.id = id;
    }


    public String toXml() {
        StringBuffer sb = new StringBuffer();
        sb.append("<" + MceConstants.QUESTIONGROUP_TAG_NAME);
        sb.append(" " + MceConstants.VISIBLE_QUESTIONS_ATTRIBUTE_NAME +
                "=\"" + getNumberOfVisibleQuestions() + "\">\n"); // id not in xml (lasse)

        for (Iterator iter = getQuestions().iterator(); iter.hasNext();) {
            Question q = (Question) iter.next();
            q.toXml(sb);
        }
        sb.append("</" + MceConstants.QUESTIONGROUP_TAG_NAME + ">");
        return sb.toString();
    }
}
