// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.List;
import java.util.LinkedList;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Calendar;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.io.Serializable;

public class Round implements Serializable {
    // number is the number of this round, min is the minimum amount of points required to pass the round
    private int number, min;
    // id is the unique id of this round
    private int id;
    // fraction of maximum points that is needed to pass the round
    private double min_frac;
    //private RoundTemplate template;
    private Date deadLine;
    // round-specific applet parameters, if any
    // private List additionalParameters;
    private String name;
    private Hashtable names;
    private Hashtable descriptions;
    private List exercises;
    private int uses;
    private List additionalParameters;
    private double makeupFactor;
    private Date open;
    // minimum number of submissions allowed for the exercises of this round
    // used to check if rounds submissions are limited
    private int minSubmissions = -1;

    public Round(int i) {
	this();
	number = i;
    }

    public Round() {
        exercises = new LinkedList();
        uses = 0;
        name = null;
        names = new Hashtable();
        descriptions = new Hashtable();
        min_frac = -1;
        min = -1;
        makeupFactor = -1;
    }

    public int getRoundNumber() { return number; }
    public void setRoundNumber(int n) { number = n; }

    public void setDeadLine(Date d) { deadLine = d;}
    public Date getDeadLine() { return deadLine; }

    public String getFormattedDeadLine() {
	DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	return df.format(getDeadLine());
    }

    public String getRoundName() {
        return name;
    }
    public void setRoundName(String name) {
        this.name = name;
    }

    public void addExercise(Exercise e) {
	minSubmissions = -1;
	exercises.add(e);
    }

    public void removeExercise(Exercise e) {
	minSubmissions = -1;
	exercises.remove(e);
    }

    public List getExerciseList() {
	return exercises;
    }

    public void addUse() {
	uses++;
    }

    public void removeUse() {
	uses--;
    }

    public int getUses() {
	return uses;
    }
    
    public void removeAllExercises() {
	minSubmissions = -1;
	while(exercises.size() > 0) {
	    Exercise e = (Exercise)exercises.get(0);
	    e.removeUse();
	    exercises.remove(0);
	}
    }

    public void setAdditionalAppletParameters(List l) {
	additionalParameters = l;
    }

    public List getAdditionalAppletParameters() {
	return additionalParameters;
    }

    public void addLocalizedData(String language, String name, String description) {
        // Set the fallback
        if ("en".equals(language) || this.name == null) {
            this.name = name;
        }
        
        names.put(language, name);
    	
        if (description == null) {
    	    description = "";
        }
    	descriptions.put(language, description);
    }

    public String getLocalizedName(String language) {
    	if (names.containsKey(language)) {
    	    return (String)names.get(language);
        } else {
    	    return getRoundName();
        }
    }

    public String getLocalizedDescription(String language) {
	if (descriptions.containsKey(language))
	    return (String)descriptions.get(language);
	else
	    return "";
    }

    //    public boolean isBeforeDeadLine() {
    //return getDeadLine().after(Calendar.getInstance().getTime());
    //    }

    public int getMax() {
	int max = 0;
	List exercises = getExerciseList();
	for(int i = 0; i < exercises.size(); i++) {
	    max += ((Exercise)exercises.get(i)).getMax();
	}
	return max;
    }

    /**
     * Returns the minimum amount of points required to pass this round.
     * The minimum points can be given either as an integer representing
     * the minimum, or as a fraction of the maximum points (a floating
     * point number). The minimum points when a fraction is used is
     * calculated by: max * frac
     *
     * An integer minimum must satisfy the following: 0 <= min <= max
     * 
     * A floating point minimum must satisfy the following:
     * 0 <= min <= 1
     *
     * An integer minimum is preferred over fraction. If no valid value is
     * given, 0 is returned.
     */
    public int getMin() {
	if(0 <= min && getMax() >= min) {
	    return min;
	}
	if(0 <= min_frac && 1 >= min_frac) {
	    return Math.round((float)(getMax() * min_frac));
	}  
	return 0;
    }

    public void setMin(double value) {
	if(value > 0 && value < 1) {
	    min_frac = value;
	    return;
	}
	if(value >= 0) {
	    min = Math.round((float)value);
	    return;
	}
	if(value < 0) {
	    min = -1;
	    min_frac = -1;
	}
    }

    public void setId(int i) {
	id = i;
    }

    public int getId() {
	return id;
    }

    public double getMakeupFactor() { return makeupFactor; }
    public void setMakeupFactor(double val) { 
	makeupFactor = val; 
    }

    public Date getOpen() {
	return open;
    }

    public void setOpen(Date val) {
	open = val;
    }

    public String getFormattedOpen() {
	DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss Z");
	if(getOpen() == null)
	    return "";
	return df.format(getOpen());
    }

    public void setFormattedOpen(String s) {}

    public boolean isOpen() {
	if (getOpen() == null) 
	    return true;
	return getOpen().before(Calendar.getInstance().getTime());
    }
    
    public int getMinimumSubmissions() {
	if (minSubmissions == -1) {
	    Iterator exers = getExerciseList().iterator();
	    int i = Integer.MAX_VALUE;
	    while (exers.hasNext()) {
		Exercise e = (Exercise)exers.next();
		i = Math.min(i, e.getSubs());
	    }
	    minSubmissions = i;
	}
	return minSubmissions;
    }
    
    public java.util.Iterator getSupportedLocaleIterator() {
        return names.keySet().iterator();
    }
    
    public List getSupportedLocales() {
        return new LinkedList(names.keySet());
    }
    
    public void setLocalizedDescription(String language, String description) {
        descriptions.put(language, description);
    }
}
