// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.Date;
import java.util.List;
import java.util.LinkedList;
import java.util.HashMap;
import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import cexer.util.PasswordService;

/**
 * Class Student represents a student.
 *
 * Some properties, such as student number and email may be provided by an
 * identity provider (through Shibboleth for example). To be able to override
 * those values without losing them permanently, both IdP and manual versions
 * are saved into database. Getters return the manual version if it's set and
 * the IdP version otherwise. The IpP provided values can be accessed with
 * the get...Idp() getters.
 */
public class Student implements Serializable, Comparable {
    private static final long serialVersionUID = -7217888839084728368L;
    private String id;
    private String name;
    private List courses;
    private List results;
    private HashMap personalDeadlines;
    private Course course;
    private String password;

    private String studentNumber;
    private String email;

    private String lang = DEFAULT_LANGUAGE;
    private Course defaultCourse;
    private List inactiveCourses;
    private int fontSize;

    private int accessLevel = DEFAULT_ACCESS;

    /** Used in the instructor ui */
    private boolean unregistered;    // XXX: shouldn't this be course-specific? -TA

    public static final int DEFAULT_ACCESS=0;
    public static final int INSTRUCTOR_ACCESS=1;
    public static final int ADMIN_ACCESS=2;

    public static final String DEFAULT_LANGUAGE = "en";

    public Student(String id) {
        this.id = id;
        studentNumber = null;
        email = null;
        
        unregistered = false;
        courses = new LinkedList();
        results = new LinkedList();
        personalDeadlines = new HashMap();
        fontSize = 12;
        inactiveCourses = new LinkedList();
    }

    public Student(String nbr, String name) {
        this(nbr);
        
        this.setName(name);
    }

    public void setFontSize(int value) {
        fontSize = value;
    }

    public int getFontSize() {
        return fontSize;
    }

    public List getCourseList() {
        return courses;
    }

    public void setCourseList(List l) {
        courses = l;
    }

    public Course getDefaultCourse() {
        return defaultCourse;
    }

    public void setDefaultCourse(Course c) {
        if (isOnCourse(c) || c == null) {
            defaultCourse = c;
        }
    }

    public Course getCourse() {
        return course;
    }

    public void setCourse(Course c) {
        course = c;
    }

    public void addCourse(Course c) {
        if(!courses.contains(c)) {
            courses.add(c);
        }
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getId() {
        return id;
    }

    public String getStudentNumber() {
        return studentNumber;
    }

    public void setStudentNumber(String studentNumber) {
        this.studentNumber = studentNumber;
    }

    public void setName(String value) {
        if (value != null) {
            this.name = value;
        } else {
            this.name = "";
        }
        
    }

    public String getName() {
        return name;
    }

    public void setPassword(String value) {
        password = value;
    }

    public String getPassword() {
        return password;
    }

    public boolean setResult(Course c, int round, int exercise, int points) {
        if(courses.contains(c)) {
            Results res = getResults(c);
            if(res == null) {
                res = new Results(c);
                results.add(res);
            }
            res.setResult(round,exercise,points);
            return true;
        } else {
            // We can't set results for a course the student
            // isn't subscribed to
            return false;
        }
    }

    /**
     * Changes the points for a specified submission. Used for modifying student points in Monitor interface
     * by cexer.components.ExerciseSubmissions
     * @param c
     * @param r
     * @param ex
     * @param points
     * @return true if modification succeeded, false otherwise
     */
    public boolean editResult(Course c, Round r, Exercise ex, int points) {
        if(courses.contains(c) && r != null && ex != null) {
            Results res = getResults(c);
            if(res == null) {
                res = new Results(c);
                results.add(res);
            }

            int subs = getSubmissions(c, r, ex);
            if(points < 0 || subs == 0) {
                return false;
            } else {
                res.setResults(r.getRoundNumber(),ex.getNumber(),points,subs);
                return true;
            }
        } else {
            // We can't set results for a course the student
            // isn't subscribed to
            return false;
        }
    }

    public Results getResults(Course c) {
        if (!courses.contains(c)) {
            return null;
        }

        for(int i = 0; i < results.size();i++) {
            Results r = (Results)results.get(i);
            if(r.getCourse().equals(c))
            return r;
        }

        // Registered for a course but not yet results -> add results
        Results res = new Results(c);
        results.add(res);

        return res;
    }

    public int getResult(Course c, Round r, Exercise e) {
        Results res = getResults(c);
        if (res == null) {
            //System.out.println("Null results returned");
            return 0;
        }

        return res.getResult(r.getRoundNumber(), e.getNumber());
    }

    public int getSubmissions(Course c, Round r, Exercise e) {
        Results res = getResults(c);
        if (res == null) {
            //System.out.println("Null results returned");
            return 0;
        }

        return res.getSubmissions(r.getRoundNumber(), e.getNumber());
    }

    public int getSubmissions(Course c, Round r) {
        Results res = getResults(c);
        if (res == null) {
            return 0;
        }

        return res.getTotalSubmissions(r.getRoundNumber());
    }

    public int getRoundResult(Course c, int round) {
        Results r = getResults(c);
        if (r == null) {
            return 0;
        }

        return r.getRoundResult(round);
    }

    public int getCourseResult(Course c) {
        if (c == null) {
            return 0;
        }

        Results r = getResults(c);
        List rounds = c.getRoundList();
        int result = 0;

        for(int i=1; i<= rounds.size();i++) {
            result += getRoundResult(c,i);
        }

        return result;
    }

    public String getGrade(Course c) {
        Results r = getResults(c);
        List rounds = c.getRoundList();
        boolean allPassed = true;

        for(int i=1; i<= rounds.size();i++) {
            Round round = (Round)rounds.get(i-1);
            
            if(getRoundResult(c,i) < round.getMin()) {
                allPassed = false;
                break;
            }
        }

        if(allPassed == false) {
            return "0";
        }

        return c.getGrade(getCourseResult(c));
    }

    public void addDeadLine(int rid, Date dl) {
        personalDeadlines.put(new Integer(rid), dl);
    }

    public String getFormattedDeadLine(int rid) {
        DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
        return df.format(getDeadLine(rid));
    }

    public boolean hasPersonalDeadLine(int rid) {
        return (getDeadLine(rid) != null);
    }

    public Date getDeadLine(int rid) {
        return (Date)personalDeadlines.get(new Integer(rid));
    }

    public List getResults() {
        return results;
    }

    /** Returns true, if the student is on the given course. */
    public boolean isOnCourse(Course c) {
        return courses.contains(c);
    }


    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public void setLanguage(String value) {
        if (value != null && !value.trim().equals("")) {
            lang = value;
        }
    }

    public String getLanguage() {
        return lang;
    }

    public void setAccessLevel(int i) {
        accessLevel = i;
    }

    public int getAccessLevel() {
        return accessLevel;
    }

    public boolean isInstructor() {
        if(accessLevel >= INSTRUCTOR_ACCESS) {
            return true;
        }

        return false;
    }

    public boolean isAdmin() {
        //System.out.println("Accesslevel: " + accessLevel);
        if(accessLevel >= ADMIN_ACCESS) {
            return true;
        }

        return false;
    }

    public boolean equals(Object o) {
        if(!(o instanceof Student)) {
            return false;
        }

        Student stu = (Student)o;
        return (getStudentNumber().equals(stu.getStudentNumber()) && getAccessLevel() == stu.getAccessLevel());
    }

    public boolean isUnregistered() {
        return unregistered;
    }

    public void setUnregistered(boolean b) {
        unregistered = b;
    }

    public void setInactiveCourses(List list) {
        inactiveCourses = list;
    }

    public synchronized void setInactive(Course c) {
        if (!inactiveCourses.contains(c)) {
            inactiveCourses.add(c);
        }

        courses.remove(c);
    }

    public synchronized void setActive(Course c) {
        inactiveCourses.remove(c);
        if (!courses.contains(c)) {
            courses.add(c);
        }
    }

    public synchronized boolean isActive(Course c) {
        return courses.contains(c);
    }

    public List getAllCourses() {
        List l = new LinkedList(courses);
        l.addAll(inactiveCourses);
        return l;
    }

    public int compareTo(Object anotherStudent) throws ClassCastException {
        if (!(anotherStudent instanceof Student)) {
            throw new ClassCastException("A Student object expected.");
        }

        return getName().compareToIgnoreCase(((Student)anotherStudent).getName());
    }

    public boolean matchPassword(String password) {
    	if (password == null){
    		return false;
    	}
        if (this.password.equals(password)) {
            return true;
        } else {
            PasswordService passwordService = PasswordService.getInstance();
            return this.password.equals(passwordService.encrypt(password));
        }
    }

    public String toString() {
        return getId();
    }
    
    /**
     * Get the student's random seed
     * @return random seed
     */
    public int getRandomSeed(){
    	return id.hashCode();
    }
}
