// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * 
 */
package cexer;
import org.apache.tapestry.BaseSessionStoreOptimized;
import cexer.util.Logging;


/**
 * This is the session-object for the student interface. It is used to store all
 * information concerning a certain visit to the TRAKLA2 web application. <br>
 * StudentSession.java and MonitorSession.java replace Visit.java in the new
 * TRAKLA2-revision (summer 2006).<br>
 * 
 * @author cefallen
 * 
 */
public class StudentSession extends BaseSessionStoreOptimized {

    /**
     * 
     */
    private static final long serialVersionUID = 4101108883591731706L;

    private static int sessionCount = 0;

    // Authentication methods
    public static final int PASSWORD = 0;
    public static final int SHIBBOLETH = 1;

    private String userId;

    /**
     * The student's name
     */
    private String name;
   
    private String password;

    private Exercise exercise;

    private Course course;

    private Round round;
    
    private boolean isLoggedIn;

    private int authMethod = 0;
    
    private String logoutUrl;
    
    /**
     * Boolean value telling if the student actually is a instructor, using the monitor interface and just having the 
     * student interface opened in another window. This is related to the logout-procedure, because a real logout from the 
     * student interface would also kill the monitor session.
     */
    private boolean usingMonitorUI;
    
    public StudentSession() {
        isLoggedIn = false;
        usingMonitorUI = false;
        logoutUrl = null;
        //System.out.println("Studentsession created");
        //throw new ApplicationRuntimeException("Studentsession created");
    }

    public void closeSession() {
        clearFields();
        StudentSession.removeSession();
    }

    private static void addSession() {
        sessionCount++;
    }

    private static void removeSession() {
        if (sessionCount > 0) {
            sessionCount--;
        }
    }
    
    /*
     * Returns the number of active student sessions. Used to measure server activity.
     * Accuracy is not guaranteed.
     * @return int Number of active sessions.
     */
    public static int getSessionCount() {
        return sessionCount;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String value) {
        userId = value;
        markSessionStoreNeeded();
    }

    public boolean getIsLoggedIn() {
        return isLoggedIn;
    }
    
    /**
     * Set the state of the session.
     * @param state true if the user has logged in, false otherwise
     */
    public void setIsLoggedIn(boolean state) {
        if(state) {
            isLoggedIn = true;
            StudentSession.addSession();
            //System.out.println("Student logged in. " + StudentSession.getSessionCount() + " active sessions.");
        } else {
            isLoggedIn = false;
        }
        
        markSessionStoreNeeded();        
    }
    
    public String getName() {
        return name;
    }

    public void setName(String value) {
        name = value;
        markSessionStoreNeeded();
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String value) {
        password = value;
        markSessionStoreNeeded();
    }

    /**
     * Gets the student object associated with this session
     * @return student object
     */
    public Student getStudent() {
        if (userId == null) {
            return null;
        } else if (usingMonitorUI && course != null) {
            Student stu = new Student(userId, name);
            stu.setCourse(getCourse());
            return stu;
        } else {
            Student student = DataStorage.getInstance().getStudent(userId);
            return student;
        }
    }

    public Exercise getExercise() {
        return exercise;
    }

    public void setExercise(Exercise ex) {
        exercise = ex;
        markSessionStoreNeeded();
    }

    /**
     * Clear the data fields in the session object.
     */
    public void clearFields() {
        userId = null;
        name = null;
        password = null;
        exercise = null;
        course = null;
        round = null;
        markSessionStoreNeeded();
    }

    public Course getCourse() {
        if (!isLoggedIn) {
            return null;
        }
        
        if (course == null) {
            Student stu = getStudent();
            if (stu == null) {
                return null;
            }
            
            if (stu.getDefaultCourse() != null) {
                setCourse(stu.getDefaultCourse());
            } else {
                setCourse((Course) stu.getCourseList().get(0));
            }
        }
        
        return course;
    }

    public void setCourse(Course c) {
        if (course != null && c != null && course.getId() != c.getId()) {
            course = c;
        }
        
        if (course == null && c != null) {
            course = c;
        }
        
        markSessionStoreNeeded();
    }

    public void clearCourse() {
        course = null;
        markSessionStoreNeeded();
    }

    public Round getRound() {
        return round;
    }

    public void setRound(Round r) {
        round = r;
        markSessionStoreNeeded();
    }

    /**
     * Set the usingMonitorUI field
     * @param state true if the user has logged in th
     */
    public void setUsingMonitorUI(boolean state) {
        usingMonitorUI = state;
        markSessionStoreNeeded();
    }

    public boolean getUsingMonitorUI() {
        return usingMonitorUI;
    }

    /**
     * Stores information about used authentication method. Possible values are PASSWORD and SHIBBOLETH.
     */
    public void setAuthMethod(int method) {
        this.authMethod = method;
        markSessionStoreNeeded();
    }

    /**
     * Returns used authentication method. Possible values are PASSWORD and SHIBBOLETH.
     */
    public int getAuthMethod() {
        return authMethod;
    }

    public String getLogoutUrl() {
        return logoutUrl;
    }
    
    public void setLogoutUrl(String logoutUrl) {
        this.logoutUrl = logoutUrl;
        markSessionStoreNeeded();
    }
    
}
