/**
 * 
 */
package cexer.components;

import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IRequestCycle;

import cexer.Choice;
import cexer.ChoiceGroup;
import cexer.Course;
import cexer.DataStorage;
import cexer.Exercise;
import cexer.ExerciseTemplate;
import cexer.MonitorSession;
import cexer.MultipleChoiceExerciseAnswer;
import cexer.MultipleChoiceExerciseTemplate;
import cexer.Question;
import cexer.QuestionGroup;
import cexer.Round;
import cexer.Student;
import cexer.pages.MonitorModifyDescriptionPage;
import cexer.util.Lock;
import cexer.util.mceparser.MceConstants;
import cexer.util.mceparser.MceParser;

/**
 * Component for displaying a multiple choice exercise in the TRAKLA2 framework.
 * 
 * @author cefallen
 *
 */
public abstract class MultipleChoiceExerciseTable extends BaseComponent {
    
	private static final String POSITIVE_FEEDBACK = "mcePositiveFeedback";
	private static final String NEGATIVE_FEEDBACK = "mceNegativeFeedback";
	
    private boolean questionGroupsLoaded = false;

    /**
     * Gets the current exercise object.
     * 
     * @return exercise
     */
    public abstract Exercise getExercise();
    
    /**
     * Gets the current course object.
     * 
     * @return course
     */
    public abstract Course getCourse();
    
    /**
     * Gets the current round object.
     *  
     * @return round
     */
    public abstract Round getRound();
    
    /**
     * Gets the student.
     * 
     * @return student
     */
    public abstract Student getStudent();
    
    /**
     * Gets the question index iterator value
     * 
     * @return question index
     */
    public abstract int getQuestionIndex();
    
    /**
     * Gets the question group index iterator value.
     * 
     * @return question group index
     */
    public abstract int getQuestionGroupIndex();
    
    /**
     * Gets the choice index iterator value.
     * 
     * @return choice index
     */
    public abstract int getChoiceIndex();
    
    /**
     * Gets the choice group index iterator value.
     * 
     * @return choice group index
     */
    public abstract int getChoiceGroupIndex();
    
    /**
     * Gets the currently active question group.
     * 
     * @return question group
     */
    public abstract QuestionGroup getQuestionGroup();
    
    /**
     * Gets the currently active question.
     * 
     * @return question
     */
    public abstract Question getQuestion();
    
    /**
     * Gets the currently active choice group.
     * 
     * @return choice group
     */
    public abstract ChoiceGroup getChoiceGroup();
    
    /**
     * Gets the currently active choice.
     * 
     * @return choice
     */
    public abstract Choice getChoice();
    
    /**
     * Gets the student specific answer object.
     */
    public abstract MultipleChoiceExerciseAnswer getAnswer();

    /**
     * Sets the student specific answer object.
     * 
     * @param answer the answer object
     */
    public abstract void setAnswer(MultipleChoiceExerciseAnswer answer);        
    
    /**
     * Sets the stored answer. Used for showing exercise feedback.
     * 
     * @param answer the answer to store
     */
    public abstract void setStoredAnswer(MultipleChoiceExerciseAnswer answer);
    
    /**
     * Gets the stored answer. Used for showing exercise feedback.
     * 
     * @return stored answer
     */
    public abstract MultipleChoiceExerciseAnswer getStoredAnswer();
    
    /**
     * Tells if the feedback mode is active.
     * 
     * @return true if feedback mode, false otherwise
     */
    public abstract boolean getShowFeedback();
    
    /**
     * Sets the feedback mode.
     * 
     * @param state true for feedback mode, false otherwise.
     */
    public abstract void setShowFeedback(boolean state);
        
    public abstract void setShowError(boolean state);
        
    /**
     * Tells if the exercise should be shown in monitor mode (all questions and choices visible).
     * 
     * @return true if monitor mode, false otherwise
     */
    public abstract boolean getIsMonitor();
        
    /**
     * Sets the saved hash string value.
     * Used for checking submission validity.
     * 
     * @param s hash string
     */
    public abstract void setSavedHashString(String s);
    
    /**
     * Gets the saved hash string value.
     * Used for checking submission validity.
     * 
     * @return hash string
     */
    public abstract String getSavedHashString();
    
    /**
     * Gets the locale object provided as component parameter. (If used).
     * 
     * @return locale
     */
    public abstract Locale getParameterLocale();
    
    public abstract MonitorSession getMonitorSession();
    
    public abstract void setExerciseInfoText(String message);
    
    public abstract void setExerciseInfo(boolean state);

    public abstract void setExerciseInfoStyle(String style);
    
    public abstract boolean getEditingEnabled();

    public abstract void setEditingEnabled(boolean enabled);
    
    public abstract String getEditXmlString();
    
    public abstract void setEditXmlString(String s);
    
    public Locale getLocale() {
        Locale parameterLocale = getParameterLocale(); 
        if(parameterLocale != null)
            return parameterLocale;
        else return getPage().getLocale();
    }
    
    /**
     * Gets all question groups of this exercise.
     * 
     * @return question groups
     */
    public List getQuestionGroups() {
    	
    	// at the first time load questiongroups from database to update possible changes made from monitor
    	if (!questionGroupsLoaded){
	        DataStorage ds = DataStorage.getInstance();
	        List groups = ds.getDatabaseMultipleChoiceQuestionGroups(getExercise().getExerciseTemplate().getId());
	        ((MultipleChoiceExerciseTemplate)getExercise().getExerciseTemplate()).setQuestionGroups(groups);
    		questionGroupsLoaded = true;
	        return groups;
    	}
    	else {
    		return ((MultipleChoiceExerciseTemplate)getExercise().getExerciseTemplate()).getQuestionGroups();
    	}
    }
    
    /**
     * Gets a generated question set from the currently active question group.
     * 
     * @return set of questions
     */
    public List getQuestionSet() { 
        if(getIsMonitor())
            return ((QuestionGroup)getQuestionGroups().get(getQuestionGroupIndex())).getQuestions();
        if(getShowFeedback())
            return ((QuestionGroup)getQuestionGroups().get(getQuestionGroupIndex())).getQuestionSet(getStudent().getRandomSeed(), getStoredAnswer().getSubmissions());
        else return ((QuestionGroup)getQuestionGroups().get(getQuestionGroupIndex())).getQuestionSet(getStudent().getRandomSeed(), getNumberOfSubmissionsUsed());
    }
    
    /**
     * Gets a question description in the currently used locale.
     * 
     * @return question description text
     */
    public String getQuestionDescription() {
        return getQuestion().getDescription(getLocale().getLanguage());
    }
    
    /**
     * Gets all choice groups associated with the currently selected question.
     * 
     * @return choice groups
     */
    public List getChoiceGroups() {
        return getQuestion().getChoiceGroups();
    }

    /**
     * Gets a generated choice set from the currently active choice group.
     * 
     * @return set of choices
     */
    public List getChoiceSet() { 
        if(getIsMonitor())
            return ((ChoiceGroup)getChoiceGroups().get(getChoiceGroupIndex())).getChoices();
        if(getShowFeedback())
            return getQuestion().getChoiceSet(getStudent().getRandomSeed(), getStoredAnswer().getSubmissions());
        else return getQuestion().getChoiceSet(getStudent().getRandomSeed(), getNumberOfSubmissionsUsed());
    }
    
    
    
    /**
     * Gets a choice description in the currently used locale.
     * 
     * @return choice description text
     */
    public String getChoiceDescription() {
        return getChoice().getDescription(getLocale().getLanguage());
    }
    
    /**
     * Tells if the current question is of radio button type. This is true if the user has to 
     * select one (and only) one of the available choices.
     * 
     * @return true if the question has radio button choices.
     */
    public boolean isRadioButtonChoices() {
        return getQuestion().getMinSelected() == 1 && getQuestion().getMaxSelected() == 1;
    }
    
    /**
     * Gets a initialized answer instance.
     * 
     * @return answer
     */
    public MultipleChoiceExerciseAnswer getInitializedAnswer() {
        if(getAnswer() == null) {
            setAnswer(new MultipleChoiceExerciseAnswer(((MultipleChoiceExerciseTemplate)getExercise().getExerciseTemplate()).getQuestionGroups(), getStudent().getRandomSeed(), getNumberOfSubmissionsUsed()));
        }
        return getAnswer();
    }
    
    /**
     * Gets the number of the selected option for a radio button group.
     * 
     * @return selected number
     */
    public int getRadioSelected() {        
        return getInitializedAnswer().getSelectedRadioChoice(getQuestion().getId());
    }
    
    /**
     * Sets the selected number for a radio button group.
     * 
     * @param selected the number of the selected item
     */
    public void setRadioSelected(int selected) {
        getInitializedAnswer().setSelectedRadioChoice(getQuestion().getId(), selected);
    }
    
    /**
     * Sets the selected value of a checkbox choice.
     * 
     * @param selected the value to set
     */
    public void setCheckboxSelected(boolean selected) {
        getInitializedAnswer().setCheckboxChoiceSelected(getQuestion().getId(), getChoiceNumber(), selected);
    }
    
    /**
     * Gets the selection information for a checkbox choice.
     * 
     * @return the value of the checkbox
     */
    public boolean getCheckboxSelected() {
        return getInitializedAnswer().getCheckboxChoiceSelected(getQuestion().getId(), getChoiceNumber());
    }
    
    /**
     * Gets the absolute (ungrouped) number of a choice in relation to a question. E.g. choice 1, choice 2, choice 3 and so on.
     * 
     * @return absolute number
     */
    public int getChoiceNumber() {
        int number = 0;
        if(getChoiceGroupIndex() > 0)
            for (int i = getChoiceGroupIndex() - 1; i >= 0; i--)
                number = number + ((ChoiceGroup)getChoiceGroups().get(i)).getNumberOfVisibleChoices();
        number = number + getChoiceIndex();
        return number;
    }
    
    /**
     * Gets the number of used submissions for this exercise.
     * Used for generating student-specific question and choice sets.
     * 
     * @return number of used submissions
     */
    public int getNumberOfSubmissionsUsed() {
        return getStudent().getSubmissions(getCourse(), getRound(), getExercise());                
    }
    
    /**
     * Tells if a student has used all his submissions on this exercise
     * 
     * @return true if all submissions are used
     */
    public boolean isAllSubmissionsUsed() {
        return getStudent().getSubmissions(getCourse(), getRound(), getExercise()) > getExercise().getSubs();
    }    
    
    /**
     * Gets the feedback text for the current question. 
     * Positive feedback is given if the question is correctly answered,
     * otherwise the negative string is returned.
     * 
     * @return feedback text
     */
    public String getQuestionFeedback() {
        if(getInitializedAnswer().getQuestionPoints(getQuestion()) 
                == getQuestion().getMaxPoints(getInitializedAnswer().getRandomSeed(), getInitializedAnswer().getSubmissions()))
            return getQuestion().getPositiveFeedback(getLocale().getLanguage());
        else return getQuestion().getNegativeFeedback(getLocale().getLanguage());
    }
    
    /**
     * Get the text representing earned points and maximum point of current question
     * @return question points text
     */
    public String getQuestionPointsText(){
    	int points = getInitializedAnswer().getQuestionPoints(getQuestion());
    	int maxPoints = getQuestion().getMaxPoints(getInitializedAnswer().getRandomSeed(), getInitializedAnswer().getSubmissions());
    	if (getLocale().getLanguage().equals("fi")){
    		return "Pisteet: " + points  + " / " + maxPoints;
    	}
    	return "Points: " + points + " / " + maxPoints;
    }
    
    
    /**
     * Gets the css style name for the feedback.
     * 
     * @return feedback style, "mcePositiveFeedback" or "mceNegativeFeedback"
     */
    public String getFeedbackStyle() {
        if(getInitializedAnswer().getQuestionPoints(getQuestion()) 
                == getQuestion().getMaxPoints(getInitializedAnswer().getRandomSeed(), getInitializedAnswer().getSubmissions()))
            return POSITIVE_FEEDBACK;
        else return NEGATIVE_FEEDBACK;
        
    }
    
    /**
     * Gets the feedback text for the current choice. 
     * 
     * @return feedback text
     */
    public String getChoiceFeedback() {
    	
    	Choice choice = getChoice();
    	if (isSelectedChoice()){
    		return choice.getSelectedFeedback(getLocale().getLanguage());    		
    	}
    	else {
    		return choice.getUnselectedFeedback(getLocale().getLanguage());
    	}
    }
    
    /**
     * Gets the selected feedback text for the current choice in current locale. 
     * 
     * @return feedback text
     */
    public String getSelectedChoiceFeedback() {
    	
    	Choice choice = getChoice();
   		return choice.getSelectedFeedback(getLocale().getLanguage());
    }

    /**
     * Gets the positive feedback text for the current choice in current locale. 
     * 
     * @return feedback text
     */
    public String getUnselectedChoiceFeedback() {
    	
    	Choice choice = getChoice();
   		return choice.getUnselectedFeedback(getLocale().getLanguage());
    }
    
    /**
     * Gets the css style name for the current choice feedback.
     * 
     * @return feedback style, "mcePositiveFeedback" or "mceNegativeFeedback"
     */
    public String getChoiceFeedbackStyle() {
    	
    	if (isSelectedChoice()){
    		return POSITIVE_FEEDBACK;
    	}
    	else {
    		return NEGATIVE_FEEDBACK;    		
    	}
    }
    
    /**
     * @return the css style name for the positive feedback
     */
    public String getPositiveFeedbackStyle(){
    	return POSITIVE_FEEDBACK;
    }
    
    /**
     * @return the css style name for the negative feedback
     */
    public String getNegativeFeedbackStyle(){
    	return NEGATIVE_FEEDBACK;
    }
    
    /**
     * @return true if the current choice is selected by user
     */
    private boolean isSelectedChoice(){
    	// radiobuttons
    	if (isRadioButtonChoices()){
	    	// selected
	    	if (getChoiceIndex() == getRadioSelected()){
	    		return true;
	    	}
	    	else {
	    		return false;
	    	}
    	}
    	//checkboxes
    	else if (getCheckboxSelected()){
    		return true;
   		}
    	else {
    		return false;
    	}
    }
    
    /**
     * Gets the points given for this exercise answer.
     * 
     * @return points
     */
    public int getExercisePoints() {
        return (int)Math.floor((float)((float)getInitializedAnswer().getPoints() 
                / (float)getInitializedAnswer().getMaxPoints()) * (float)getExercise().getMax()); 
    }
    
    /**
     * Submits the exercise answer for grading.
     * 
     * @param cycle the request cycle
     */
    public void formSubmit(IRequestCycle cycle) {
        if(!getHashString().equals(getSavedHashString())) {
            setShowError(true);
            createNewExerciseInstance(cycle);
            return;
        }
        
        if (validSelectionCount()){
	        int exercisePoints = (int)Math.floor((float)((float)getInitializedAnswer().getPoints() 
	                / (float)getInitializedAnswer().getMaxPoints()) * (float)getExercise().getMax());        
	        DataStorage ds = DataStorage.getInstance();
	
	        setStoredAnswer(getAnswer());
	        getStoredAnswer().setSubmitted(true);
	        getStoredAnswer().setExerciseId(getExercise().getExerciseTemplate().getId());
	        setShowFeedback(true);
	
	        int status = DataStorage.OK;
	        if(!isBeforeDeadline(getCourse(), getRound(), getStudent()))
	            status = DataStorage.AFTER_DEADLINE;
	
	        int result = ds.setResult(getStudent().getId(), Integer.toString(getCourse().getId()), 
	                getRound().getRoundNumber(), getExercise().getNumber(), exercisePoints, "", status);
	
	        if(result != DataStorage.OK)
	            System.out.println("Error adding result to database!");
	        cycle.activate(getPage());
        }
    }
    
    /**
     * Link used for creating a new exercise instance. The user clicks on the link
     * if he wants to try to solve the exercise one more time (if he has submissions left)
     * and the system generates a new questionaire.
     * 
     * @param cycle
     */
    public void createNewExerciseInstance(IRequestCycle cycle) {
        setAnswer(null);
        setShowFeedback(false);
        setStoredAnswer(null);
        cycle.activate(getPage());
    }
    
    /**
     * Helper function for <code>formSubmit</code>. Tells if the submission is made before deadline.
     * 
     * @param c current course
     * @param r current round
     * @param s student
     * @return true if the submission is before the deadline, false otherwise
     */
    private boolean isBeforeDeadline(Course c, Round r, Student s) {
        Calendar deadLine = Calendar.getInstance();
        if (s.hasPersonalDeadLine(r.getId()))
            deadLine.setTime(s.getDeadLine(r.getId()));
        else
            deadLine.setTime(r.getDeadLine());
        if (deadLine.getTime().after(Calendar.getInstance().getTime()))
            return true;
        else
            return false;
    }
    
    /**
     * Gets the question group formatting. Only used when the exercise is viewed in the monitor interface.
     * 
     * @return the css class used for formatting the question group
     */
    public String getQuestionGroupClass() {
        if(getIsMonitor())
            return "questionGroupBorder";
        else return "";
    }
    
    /**
     * Gets the choice group formatting. Only used when the exercise is viewed in the monitor interface.
     * 
     * @return the css class used for formatting the choice group
     */
    public String getChoiceGroupClass() {
        if(getIsMonitor())
            return "choiceGroupBorder";
        else return "";
    }
    
    /**
     * Gets the hash string used for checking the submission validity at form submission.
     * The string is saved in a hidden form field.
     * 
     * @return hash string
     */
    public String getHashString() {
        // XXX Hash this information with PasswordService after merging with head branch.
        // (And remove the garbage string at the beginning)
        return "adfswgfjerkglesjglk" + 100 + getNumberOfSubmissionsUsed();
    }
        
    /**
     * Sets the hash string. Used by tapestry framework magic when submitting the questionaire form.
     * Tapestry reads the hash string from a hidden form field.
     * 
     * @param s hash string
     */
    public void setHashString(String s) {
        setSavedHashString(s);
    }
    

    /**
     * @return the xml string of the exercise
     */
    public String getXml() {
    	
        StringBuffer sb = new StringBuffer();
        sb.append("<" + MceConstants.MCEXERCISE_TAG_NAME + ">\n");
        
        List groups = getQuestionGroups();
        for (int i=0; i < groups.size(); i++){
        	sb.append(((QuestionGroup)groups.get(i)).toXml() + "\n");
        }
        sb.append("</" + MceConstants.MCEXERCISE_TAG_NAME + ">\n");
        
        return sb.toString();
    }
    
    /**
     * Edits the exercise based on the updated xml.
     */
    public void editExercise(IRequestCycle cycle) {
    	
        List groups = new LinkedList();
        try {
            MceParser parser = new MceParser();
            parser.parse(getEditXml());
            groups = parser.getQuestionGroups();
        } catch (Exception e) {
            e.printStackTrace();
    	    setExerciseInfo(true);
    	    setExerciseInfoStyle(NEGATIVE_FEEDBACK);
    	    setExerciseInfoText("Error parsing the xml. Exercise was not updated.");
    	    setEditingEnabled(true);
            return;
        }
    	
        DataStorage ds = DataStorage.getInstance();
        int eid = getExercise().getExerciseTemplate().getId();
        ds.updateMultipleChoiceExercise(eid, groups);
        ExerciseTemplate et = getExercise().getExerciseTemplate();
        
        if (et instanceof MultipleChoiceExerciseTemplate){
        	((MultipleChoiceExerciseTemplate)et).setQuestionGroups(groups);
        }
	    setExerciseInfo(true);
	    setExerciseInfoStyle(POSITIVE_FEEDBACK);
	    setExerciseInfoText("Exercise updated successfully");
	    destroyLock(cycle);
	    setEditingEnabled(false);
    }
    
    /**
     * Cancels the exercise editing
     */
    public void cancelEditExercise(IRequestCycle cycle) {
	    setExerciseInfo(true);
	    setExerciseInfoStyle(POSITIVE_FEEDBACK);
	    setExerciseInfoText("Exercise editing cancelled");
	    destroyLock(cycle);
	    setEditingEnabled(false);
    }
    
    /**
     * @return the editXml or generated xml presentation of the exercise if editXml is empty 
     */
    public String getEditXml(){
    	String editXml = getEditXmlString();
    	if (editXml == null){
    		return getXml();
    	}
   		return editXml;
    }
    
    public void setEditXml(String s){
    	setEditXmlString(s);
    }
    
    /**
     * @return the info text of the exercise
     */
    public String getQuestionInfo(){
    	return getQuestion().getInfo();
    }
    
    /**
     * Start editing the exercise if no lock is added for the exercise.
     */
    public void startEditExercise(){
    	Exercise e = getExercise();
    	Integer eid = new Integer(e.getExerciseTemplate().getId());
    	DataStorage ds = DataStorage.getInstance();
    	Lock l = ds.addLock(eid, getMonitorSession());
    	if (l != null){
    		l.setType(Lock.LOCK_TYPE_MULTIPLECHOICE);
    		setEditingEnabled(true);
    	}
    }
    
    /**
     * Removes a lock from the current exercise
     * @param cycle
     */
    public void destroyLock(IRequestCycle cycle) {
		DataStorage ds = DataStorage.getInstance();
		ds.removeLock(new Integer(getExercise().getExerciseTemplate().getId()));
    }

    /**
     * Tells if the exercise is locked, eg. someone else is editing it or forgot to correctly exit the edit mode.
     * @return true if the exercise is locked
     */
    public boolean isLocked() {
		DataStorage ds = DataStorage.getInstance();
		Lock l = ds.getLock(new Integer(getExercise().getExerciseTemplate().getId()));
		return (l != null);
    }
    
    /**
     * Checks that the amount of selected choices is valid
     * @return true if correnct amount of choices is selected
     */
    private boolean validSelectionCount(){
    	
    	boolean result = true;
    	Iterator groupsIterator = getQuestionGroups().iterator();

    	while (groupsIterator.hasNext()){
    		Iterator questionsIterator = ((QuestionGroup)groupsIterator.next()).getQuestionSet(getStudent().getRandomSeed(), getNumberOfSubmissionsUsed()).iterator();
    		
    		while (questionsIterator.hasNext()){
    			Question tmpQuestion = (Question)questionsIterator.next();
    			tmpQuestion.setInfo("");
    			int maxSelected = tmpQuestion.getMaxSelected();
    			int minSelected = tmpQuestion.getMinSelected();
    			
    			// if not radiobuttons
    			if (!(maxSelected == 1 && minSelected == 1)){
    				int selectedCount = getInitializedAnswer().getSelectedCount(tmpQuestion);
    				if (selectedCount < minSelected || selectedCount > maxSelected){
    					if (getLocale().getLanguage().equals("fi")){
    						tmpQuestion.setInfo("Valitse v&auml;hint&auml;&auml;n " + minSelected + " ja enint&auml;&auml;n " + maxSelected + " vaihtoehtoa");
    					}
    					else {
    						tmpQuestion.setInfo("Select at least " + minSelected + " and up to " + maxSelected + " choices");    						
    					}
    					result = false;
    				}
    			}
    		}
    	}
    	return result;
    }
}
