// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * 
 */
package cexer.pages;

import cexer.Course;
import cexer.DataStorage;
import cexer.StudentSession;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import cexer.Student;
import cexer.util.CourseSelectionModel;
import cexer.util.Logging;
import cexer.util.MailerService;

import org.apache.hivemind.Messages;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;

/**
 * @author cefallen
 *
 */
public abstract class FeedbackPage extends SecurePage {

    /**
     * Tapesty 4 application state object. Fetches the injected student-session
     * object (see Login.page)
     */

    public abstract StudentSession getStudentSession();
        
    /**
     * Asks the framework if the StudentSession-object exists, e.g. if the HttpSession has been created
     * or if the current session is stateless.
     * @return TRUE if the StudentSession-object is created, otherwise FALSE
     */
    public abstract boolean getSessionState();
    
    public abstract HttpServletRequest getRequest();
    
    public abstract String getFreeResponse();
    
    public abstract String getException();
    
    public abstract String getName();
    
    public abstract String getUserid();
    
    public abstract String getEmail();
    
    public abstract boolean getAnonymousFeedback();
    
    public abstract Course getSelectedCourse();
    
    public abstract int getSelectedExercise();
    
    public abstract int getSelectedRound();
    
    public abstract String getSelectedConnection();
    
    public IPropertySelectionModel getRoundSelectionModel() {
        return new IntegerSelectionModel(10);
    }
    
    public IPropertySelectionModel getExerciseSelectionModel() {
        return new IntegerSelectionModel(15);
    }

    /**
     * @return A CourseSelectionModel containing the courses that the user has enrolled to, or all courses
     * if the user hasn't logged in.
     */
    public IPropertySelectionModel getCourseSelectionModel() {
        DataStorage ds = DataStorage.getInstance();
        if(getSessionState() && getStudentSession().getStudent() != null) {
            Student s = getStudentSession().getStudent();
            return new CourseSelectionModel(s.getCourseList());
        } else {
            return ds.getCourseSelectionModel();
        }
    }
    
    public IPropertySelectionModel getConnectionSelectionModel() {
        Messages messages = getMessages();
        String[] array = new String[6];
        array[0] = messages.getMessage("connection.direct.text");
        array[1] = messages.getMessage("connection.wireless.text");
        array[2] = messages.getMessage("connection.isdn.text");
        array[3] = messages.getMessage("connection.modem.text");
        array[4] = messages.getMessage("connection.gprs.text");
        array[5] = messages.getMessage("connection.other.text");
        return new StringPropertySelectionModel(array);        
    }
                    
    public boolean isLoggedIn() {
        return (getSessionState() && getStudentSession().getIsLoggedIn());
    }
    
    public abstract void setFeedbackSent(boolean value);
    public abstract boolean getFeedbackSent();
    
    public void submitFeedback(IRequestCycle cycle) {        
        HttpServletRequest request = getRequest();
        String message = "User feedback from the feedback form:\n ";
        if(getSessionState() && getStudentSession().getStudent() != null && !getAnonymousFeedback()) {
            message += "\nUser id: " + getStudentSession().getStudent().getId();
            message +="\nName: " + getStudentSession().getStudent().getName();
            message +="\nEmail: " + getStudentSession().getStudent().getEmail();
            message +="\nConnecting from host " + request.getRemoteHost() + " (" + request.getRemoteAddr() + ")";
        } else if(!getAnonymousFeedback()) {
            message +="\nUser id: " + getUserid();
            message +="\nName: " + getName();
            message +="\nEmail: " + getEmail();
            message +="\nConnecting from host " + request.getRemoteHost() + " (" + request.getRemoteAddr() + ")";
        } else
            message +="\nFeedback by an anonymous user.";
        message += "\nConnection method: " + getSelectedConnection();
        message += "\nUser-Agent: " + request.getHeader("User-Agent");
        message += "\nCourse: " + getSelectedCourse().getFullCourseName();
        if(getSelectedRound() > 0)
            message += "\nRound: " + getSelectedRound(); 
        if(getSelectedExercise() > 0)
            message += "\nExercise: " + getSelectedExercise();
        message += "\n\nMessage:\n" + getFreeResponse();        
        if(getException() != null && getException().length() > 0)
            message += "\n\nException:\n" + getException();
        message += "\n\n";
        MailerService.getInstance().mailToFeedbackList(message);
        setFeedbackSent(true);
        cycle.activate(this);
    }
    
    private class IntegerSelectionModel implements IPropertySelectionModel {
        private int maxpoints;

        public IntegerSelectionModel(int maxpoints) {
            this.maxpoints = maxpoints;
        }

        public String getLabel(int index) {
            if(index > 0)
                return Integer.toString(index) + " ";
            else return " ";
        }

        public Object getOption(int index) {
            return new Integer(index);
        }

        public int getOptionCount() {
            return maxpoints + 1;
        }

        public String getValue(int index) {
            return Integer.toString(index);
        }

        public Object translateValue(String value) {
            return getOption(Integer.parseInt(value));
        }
    }
}