// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import org.apache.tapestry.IRequestCycle;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import cexer.*;
import cexer.util.LocaleSelectionModel;
import cexer.util.Lock;
import cexer.util.Logging;

public abstract class MonitorTestExercisePage extends SecurePage {
    
    private int appletWidth, appletHeight;

    /**
     * Gets the monitor interface user session object
     * @return the session object
     */
    public abstract MonitorSession getMonitorSession();
    
    /**
     * Set the warning property for the page, if true the content of the 
     * warningText-property will be displayed
     * @param boolean state of warning
     */
    public abstract void setWarning(boolean state);
    
    /**
     * Sets the warning text
     * @param text
     */
    public abstract void setWarningText(String text);
    
    public abstract Locale getLocaleToAdd();
    
    public abstract String getLocalizedExerciseNameToAdd();
    
    /**
     * Link to the MonitorModifyDescription-page, takes the user into edit mode 
     * for the exercise description in selected locale
     * @param cycle
     */
    public void modifyDescription(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
    	Exercise e = session.getExercise();
    	Integer eid = new Integer(e.getExerciseTemplate().getId());
    	DataStorage ds = DataStorage.getInstance();
    	Lock l = ds.addLock(eid, getMonitorSession());
    	if(l == null) {
    	    cycle.activate(this);
    	    return;
    	} else {
    	    MonitorModifyDescriptionPage nextPage = (MonitorModifyDescriptionPage) cycle.getPage("MonitorModifyDescription");
            Object[] params = cycle.getListenerParameters();
            nextPage.setTheLocale((String) params[0]);
    	    cycle.activate(nextPage);
    	}
    }
    
    public void localizeStrings(IRequestCycle cycle) {
        MonitorLocalizeAppletPage nextPage = (MonitorLocalizeAppletPage)cycle.getPage("MonitorLocalizeApplet");
        
        Course course = getMonitorSession().getCourse();
        
        if (course != null && course.getId() != DataStorage.DUMMY_COURSE) {
            nextPage.setCourseId(course.getId());
        } else {
            nextPage.setCourseId(0);
        }
        
        nextPage.setLanguage((String)cycle.getListenerParameters()[0]);
        nextPage.setExerciseId(getMonitorSession().getExercise().getExerciseTemplate().getId());
        cycle.activate(nextPage);
    }
    
    
    /**
     * Returns a LocaleSelectionModel containing the locales in which the exercise doesn't have a description.
     * @return LocaleSelectionModel with the missing locales
     */
    public LocaleSelectionModel getMissingLocaleModel() {
        Iterator i = getLocales();
        DataStorage ds = DataStorage.getInstance();
        List missingLocales = ds.getLocales();
        //System.out.println("ds.getLocales().size(): " + missingLocales.size());
        while(i.hasNext()) {
            missingLocales.remove(new Locale((String)i.next()));
        }
        return new LocaleSelectionModel(missingLocales);                                    
    }    
    
    /**
     * Tells if the exercise is missing a description in one (or more) of the locales specified in
     * DataStorage by getLocaleSelectionModel
     * @return true if exercise doesn't have descriptions in all available locales
     */
    public boolean isMissingLocales() {
        Iterator i = getLocales();
        DataStorage ds = DataStorage.getInstance();
        int size = 0;
        while(i.hasNext()) {
            size++;
            i.next();
        }
        if(size < ds.getLocaleSelectionModel().getOptionCount())
            return true;
        return false;
    }
    
    /**
     * Adds a empty description in the selected locale
     * @param cycle
     */
    public void addDescription(IRequestCycle cycle) {
        if(getLocaleToAdd() == null || getLocalizedExerciseNameToAdd() == null) {
            cycle.activate(this);
            return;
        }
        DataStorage ds = DataStorage.getInstance();
        boolean success = ds.addExerciseData(getLocalizedExerciseNameToAdd(), "", getLocaleToAdd().getLanguage(), getMonitorSession().getExercise().getExerciseTemplate().getId(), 0);
        if(success) {
            getMonitorSession().getExercise().getExerciseTemplate().addLocalizedData(getLocaleToAdd().getLanguage(), getLocalizedExerciseNameToAdd(), "");            
        }
        cycle.activate(this);
    }
    

    /**
     * Removes a lock from the current exercise
     * @param cycle
     */
    public void destroyLock(IRequestCycle cycle) {
	DataStorage ds = DataStorage.getInstance();
    MonitorSession session = getMonitorSession();
	ds.removeLock(new Integer(session.getExercise().getExerciseTemplate().getId()));
	cycle.activate(this);
    }

    /**
     * Tells if the exercise description is locked, eg. someone else is editing it or forgot to correctly exit the edit mode.
     * @return true if the exercise description is locked
     */
    public boolean isLocked() {
	DataStorage ds = DataStorage.getInstance();
    MonitorSession session = getMonitorSession();
	Lock l = ds.getLock(new Integer(session.getExercise().getExerciseTemplate().getId()));
	return (l != null && l.getId() != session && l.getType() == Lock.LOCK_TYPE_DESCRIPTION);
    }

    /**
     * @return Iterator for locales supported by the exercise
     */
    public Iterator getLocales() {
        return getMonitorSession().getExercise().getSupportedLocales().iterator();
    }

    public abstract String getTheLocale();
    public abstract void setTheLocale(String s);

    /**
     * Get the name of the exercise in current language
     * @return String contaning name
     */
    public String getName() {
	String loc = getTheLocale();
	return getMonitorSession().getExercise().getLocalizedName(loc);
    }

    /**
     * Get the exercise description in old unformatted format (without tabs)
     * @return String contaning old style description
     */
    public String getDescription() {
	String loc = getTheLocale();
	return getMonitorSession().getExercise().getLocalizedDescription(loc);
    }    
    
    /**
     * Tells if the exercise description is in new (tabbed) format
     * @return true if the description is organized into tabs
     */
    public boolean getHasNewDescription() {
        if(getMonitorSession().getExercise().getLocalizedTask(getTheLocale()).equals(""))
            return false;
        return true;
    }

    /**
     * @return String containing the task tab content in current language
     */
    public String getTask() {
        return getMonitorSession().getExercise().getLocalizedTask(getTheLocale());
    }
    
    /**
     * @return String containing the instructions tab content in current language
     */
    public String getInstructions() {
        return getMonitorSession().getExercise().getLocalizedInstructions(getTheLocale());
    }
    
    /**
     * @return true if the exercise description has a instrcutions-tab
     */
    public boolean getHasInstructions() {
        if(!getInstructions().equals("")) {
            return true;
        }
        return false;
    }
    
    /**
     * 
     * @return true if the exercise has pseudocode blocks
     */
    public boolean getHasCodeBlocks() {
        if(getMonitorSession().getExercise().getLocalizedCodeBlockNames(getTheLocale()).size() > 0) {
            //System.out.println("Exercise has " + getExercise().getLocalizedCodeBlockNames(getLocale().getLanguage()).size() + "code blocks");
            return true;
        }
        return false;
    }
    
    /**
     * Get the names of the pseudocode blocks 
     * @return LinkedList containing the names in current languge
     */
    public LinkedList getCodeBlockNames() {
        return getMonitorSession().getExercise().getLocalizedCodeBlockNames(getTheLocale());        
    }
    
    /**
     * Get the pseudocode block (content) with specified name
     * @param blockName
     * @return String containing the pseudocode
     */
    public String getCodeBlock(String blockName) {
        return getMonitorSession().getExercise().getLocalizedCodeBlock(blockName, getTheLocale());          
    }

    /**
     * @return true if the exercise has a hintpage
     */
    public boolean isHintPage() {
        String hintPage = getMonitorSession().getExercise().getHintPage();
        return !(hintPage == null || hintPage.equals(""));
    }
    
    public abstract int getIndex();
    public abstract int getTabIndex();
    public abstract int getTabTextIndex();
    public abstract String getCodeBlockName();
    
    /**
     * Get the css class for pseudocode tabs. The first tab has style "current"
     * all other tabs have a neutral style (unselected)
     * @return style information for pseudocode tabs
     */
    public String getCodeBlockTabClass() {
        if(getTabIndex() == 0)
            return "current";
        return "";
    }
    
    /**
     * Get the javascript action associated with a psedocode block (onclick)
     * @return
     */
    public String getCodeBlockTabAction() {
        return "showTab('" + getTheLocale() + "code', " + getTabIndex() +  "); return false;";
    }
    
    /**
     * Get the css styles for pseudocode blocks. The first block is displayed normally
     * all other blocks are hidden with style "display: none"
     * @return style information for pseudocode blocks
     */
    public String getCodeBlockTextStyle() {
        if(getTabTextIndex() != 0)
            return "display: none";
        else return "";
    }
    
    /**
     * Get the unique element id for a pseudocode block
     * @return String containing the element id
     */
    public String getCodeBlockId() {
        return getCodeBlockName().toLowerCase() + getIndex() + "-text";
    }
    
    /**
     * Get the exercise applet width.
     * @return
     */
    public int getWidth() {
        if(isExerciseHasApplet())
            return ((AppletExerciseTemplate)getMonitorSession().getExercise().getExerciseTemplate()).getWidth();
        else return 0;
    }
    
    
    /**
     * Get the exercise applet height
     * @return
     */
    public int getHeight() {
        if(isExerciseHasApplet())
            return ((AppletExerciseTemplate)getMonitorSession().getExercise().getExerciseTemplate()).getHeight();
        else return 0;
    }    
    
    public int getNewHeight() {
        return getHeight();
    }
    
    public int getNewWidth() {
        return getWidth();
    }        
    
    public void setNewHeight(int height) {
        appletHeight = height;
    }

    public void setNewWidth(int width) {
        appletWidth = width;
    }
    
    /**
     * Changes the applet width and height to the values specified in the form.
     * @param cycle
     */
    public void changeAppletSize(IRequestCycle cycle) {
        DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
        if(appletWidth < 200 || appletHeight < 200) {
            setWarning(true);
            setWarningText("Applet height or width can't be less than 200px.");
            return;
        }            
        boolean success = ds.setExerciseAppletSize(session.getExercise().getExerciseTemplate().getId(), appletWidth, appletHeight);
        if(success) { //if successfully changed in database, also make the changes in the logic layer objects
            ((AppletExerciseTemplate)session.getExercise().getExerciseTemplate()).setHeight(appletHeight);
            ((AppletExerciseTemplate)session.getExercise().getExerciseTemplate()).setWidth(appletWidth);           
        }
    }
    
    /**
     * Link to the MonitorAllExercises page
     * @param cycle
     */
    public void prevPage(IRequestCycle cycle) {
	if(getMonitorSession().getRound().getId() > 0) {
	    cycle.activate("MonitorCourseInformation");
	    return;
	}
	cycle.activate("MonitorAllExercises");
    }

    /**
     * Tells if the exercises are browsed from a course (return false) or from the all exercises list (return true)
     * @return true if the current course is a dummy course
     */
    public boolean isDummy() {
	DataStorage ds = DataStorage.getInstance();
	return getMonitorSession().getCourse().getId() == DataStorage.DUMMY_COURSE;
    }

    /**
     * @return true if there is a exercise before the current one
     */
    public boolean isPrevious() {
        MonitorSession session = getMonitorSession();
        Round r = session.getRound();
        Exercise e = session.getExercise();
        List exercises = r.getExerciseList();
        int i = exercises.indexOf(e);
        if (i < 0)
            return false;
        if (i == 0) {
            List rounds = session.getCourse().getRoundList();
            return (rounds.indexOf(r) > 0);
        }
        return true;
    }

    /**
     *  @return true if there is a exercise after the current one
     */
    public boolean isNext() {
        MonitorSession session = getMonitorSession();
        Round r = session.getRound();
        Exercise e = session.getExercise();
        List exercises = r.getExerciseList();
        int i = exercises.indexOf(e);
        if (i < 0)
            return false;
        if (i == exercises.size() - 1) {
            List rounds = session.getCourse().getRoundList();
            return (rounds.indexOf(r) < rounds.size() - 1);
        }
        return true;
    }

    /**
     * Link to previous exercise
     * @param cycle
     */
    public void previousExercise(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
	Course c = session.getCourse();
	Round r = session.getRound();
	Exercise e = session.getExercise();
	List exercises = r.getExerciseList();
	int i = exercises.indexOf(e);
	if(i < 0) {
	    // an error situation
	    return;
	}
	if(i == 0) {
	    List rounds = c.getRoundList();
	    i = rounds.indexOf(r);
	    r = (Round)rounds.get(i-1);
	    exercises = r.getExerciseList();
	    e = (Exercise) exercises.get(exercises.size()-1);
        session.setRound(r);
        session.setExercise(e);
	    cycle.activate("MonitorTestExercise");
	    return;
	}
	e = (Exercise)exercises.get(i-1);
    session.setExercise(e);
	cycle.activate("MonitorTestExercise");
	return;
    }

    /**
     * Link to next exercise
     * @param cycle
     */
    public void nextExercise(IRequestCycle cycle) {
        MonitorSession session = getMonitorSession();
	Round r = session.getRound();
	Exercise e = session.getExercise();
	List exercises = r.getExerciseList();
	int i = exercises.indexOf(e);
	if(i < 0) { // XXX this is an error
	    return; 
	}
	if(i == exercises.size() -1) {
	    List rounds = session.getCourse().getRoundList();
	    i = rounds.indexOf(r);
	    r = (Round)rounds.get(i+1);
	    exercises = r.getExerciseList();
	    e = (Exercise)exercises.get(0);
        session.setRound(r);
        session.setExercise(e);
	    cycle.activate("MonitorTestExercise");
	    return;
	}
	e = (Exercise)exercises.get(i+1);
    session.setExercise(e);
	cycle.activate("MonitorTestExercise");
	return;
    }
    
    /**
     * @return true if the Exercise has a applet
     */
    public boolean isExerciseHasApplet() {
        if(getMonitorSession().getExercise().getExerciseTemplate() instanceof AppletExerciseTemplate)
            return !((AppletExerciseTemplate)getMonitorSession().getExercise().getExerciseTemplate()).getCode().equals("");
        else return false;
    }
    
    public boolean isCourseSpecificDescription() {
        return getMonitorSession().getExercise().hasCourseSpecificDescription(getTheLocale());
    }
    
    /**
     * Tells if the exercise is a multiple choice questionaire.
     * 
     * @return true if multiple choice, false otherwise
     */
    public boolean isMultipleChoiceExercise() {
        return getMonitorSession().getExercise().getExerciseTemplate() instanceof MultipleChoiceExerciseTemplate;
    }

    public void detach() {
        super.detach();
        appletWidth = appletHeight = 0;
    }
    
    public boolean getHasAdditionalTabs() {
        if (getMonitorSession().getExercise().getLocalizedTabNames(getTheLocale()).size() > 0)
            return true;
        return false;
    }
    
    
    public List getTabNames() {
        return getMonitorSession().getExercise().getLocalizedTabNames(getTheLocale());        
    }
    
    public String getTab(String tabName) {
        return getMonitorSession().getExercise().getLocalizedTab(tabName, getTheLocale());          
    }
    
    public int getAdditionalTabIndex() {
        int firstAdditionalTabIndex = getTabIndex();
        if (getHasNewDescription())
            firstAdditionalTabIndex++;
        if (getHasInstructions())
            firstAdditionalTabIndex++;
        return firstAdditionalTabIndex;
    }
    
    public String getTabAction() {
        return "showTab('" + getTheLocale() + "tab', " + getAdditionalTabIndex() +  "); return false;";
    }

    public abstract String getTabName();
}
