// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.valid.ValidationDelegate;
import org.apache.tapestry.event.PageBeginRenderListener;
import org.apache.tapestry.event.PageValidateListener;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.services.RequestGlobals;
import org.apache.tapestry.annotations.InjectObject;
import org.apache.tapestry.PageRedirectException;
import cexer.*;
import cexer.util.Checker;
import cexer.util.CourseSelectionModel;
import cexer.util.Logging;
import cexer.util.LocaleSelectionModel;
import cexer.util.PasswordService;

public abstract class RegisterPage extends SecurePage implements PageBeginRenderListener {

    /*
     * Handles the GET parameter
     */
    public void pageBeginRender(PageEvent event) {
        //setLanguage(new Locale(Student.DEFAULT_LANGUAGE));

        // Handle the GET parameter
        String param = getPage().getRequestCycle().getParameter("cid");
        int cId = -1;
        
        if (param != null && param.length() > 0) {
            try {
                cId = Integer.parseInt(param);
            } catch (NumberFormatException e) {
                Logging.error(this, "Invalid course id as a parameter for RegisterPage: " + param);
            }
        }

        if (cId > 0) {
            // Make it persist
            setCourseParam(cId);

            // Check that course is open
            Course course = DataStorage.getInstance().getCourse(cId);
            if (course == null || !course.isOpen()) {
                setWarning(true);
                setWarningText("Registration is not open.");
                return;
            }
        }
    }

    public void formSubmit(IRequestCycle cycle) {
        setWarning(true);
        Course course = getCourse();
        
        // Check that course is open
        if (course == null || !course.isOpen()) {
            setWarningText("Registration is not open.");
            return;
        }
        
        // Don't proceed if the form is not filled correctly
        ValidationDelegate delegate = (ValidationDelegate)getBeans().getBean("delegate");
        if (delegate.getHasErrors()) {
            return;
        }

        DataStorage ds = DataStorage.getInstance();
        String studentNumber = getUserId().toUpperCase();
        Student student = ds.getStudent(studentNumber);
        String checker = course.getCheckerClass();

        if(!Checker.check(studentNumber, checker)) {
            setWarningText("Invalid student number for the selected course.");
            return;
        }

        if (!getPassword().equals(getPassword2())) {
            setWarningText("Passwords do not match");
            return;
        }

        if (getPassword().length() > 12) {
            setWarningText("Password too long (max 12 characters).");
            return;
        }

        if (student != null) {
            // Student exists in db
            Logging.info(this, "existing student");
            
            if (!student.matchPassword(getPassword())) {
                setWarningText("Student already registered, password incorrect.");
                Logging.info(this, "Student " + student.getStudentNumber() +
                            " tried to register to a new course with invalid password.");
                return;
            } else if (student.isOnCourse(course)) {
                setWarningText("You are already registered to the selected course.");
                return;
            }

            if (student.getName() == null || !student.getName().equals(getName())) {
                student.setName(getName());
                ds.setStudentField(studentNumber, "name", getName());
            }

            if (student.getEmail() == null || !student.getEmail().equals(getEmail())) {
                student.setEmail(getEmail());
                ds.setStudentField(studentNumber, "email", getEmail());
            }

            String lang = getLanguage().getLanguage();
            if (!student.getLanguage().equals(lang)) {
                student.setLanguage(lang);
                ds.setStudentField(studentNumber, "lang", lang);
            }
        } else {
            // New student
            Logging.info(this, "new student");
            
            PasswordService passwordService = PasswordService.getInstance();
            student = new Student(studentNumber, getName());
            student.setName(getName());
            student.setPassword(passwordService.encrypt(getPassword()));
            student.setEmail(getEmail());
            student.setStudentNumber(studentNumber);
            String lang = getLanguage().getLanguage();
            student.setLanguage(lang);

            boolean ok = ds.addStudent(student);
            if (!ok) {
                setWarningText("Error when adding student!");
                Logging.error(this, "Error when adding student " + studentNumber);
                return;
            }
            
            // Log
            cexer.util.Logging.info("Created user " + studentNumber + " (traditional)");
            ds.addLogEntry(DataStorage.CREATEUSER_TRADITIONAL, studentNumber);
        }

        // Register to course
        student.addCourse(course);
        student.setDefaultCourse(course);
        ds.setStudentDefaultCourse(studentNumber, course.getId());
        ds.registerStudentToCourse(studentNumber, course.getId());

        // Login
        StudentSession session = getStudentSession();
        session.clearFields();
        session.setIsLoggedIn(true);
        session.setAuthMethod(StudentSession.PASSWORD);
        session.setUserId(studentNumber);
        getEngine().setLocale(getLanguage());
        
        // Log
        cexer.util.Logging.info("Registered " + studentNumber + " to "
                                + course.getId() + " (traditional)");
        ds.addLogEntry(DataStorage.REGISTER_TRADITIONAL, studentNumber);

        cycle.activate("Main");
    }

    public boolean isCourseParameter() {
        return (getCourseParam() > 0);
    }


    public void detach() {
        super.detach();
        setLanguage(new Locale(Student.DEFAULT_LANGUAGE));
    }

    /**
     * Returns a list of available courses.
     */
    public CourseSelectionModel getCourses() {
        return DataStorage.getInstance().getLocalizedCourseSelectionModel(getPage().getLocale().getLanguage());
    }

    /**
     * Returns a list of available languages.
     */
    public LocaleSelectionModel getLocaleModel() {
        LocaleSelectionModel lsm = DataStorage.getInstance().getLocaleSelectionModel();
        return lsm;
    }

    public String getLocalizedCourseName() {
        Course c = getCourse();

        if(c != null) {
            return c.getCourseCode() + " " + c.getLocalizedName(getPage().getLocale().getLanguage());
        } else {
            return "";
        }
    }

    
    // Stores the GET parameter into a persistent property
    public abstract void setCourseParam(int cid);
    public abstract int getCourseParam();
    
    public abstract void setCourseDropdown(Course course);
    public abstract Course getCourseDropdown();
    
    /**
     * Returns the selected course either form GET parameter or from the dropdown.
     * GET parameter always overrides the dropdown.
     */ 
    public Course getCourse() {
        int courseParam = getCourseParam();
        
        if (courseParam > 0) {
            return DataStorage.getInstance().getCourse(courseParam);
        } else {
            return getCourseDropdown();
        }
    }
    
    public abstract StudentSession getStudentSession();

    public abstract void setPassword(String value);

    public abstract String getPassword();

    public abstract void setPassword2(String value);

    public abstract String getPassword2();

    public abstract void setEmail(String value);

    public abstract String getEmail();

    public abstract void setLanguage(Locale value);

    public abstract Locale getLanguage();

    public abstract String getName();

    public abstract void setName(String name);

    public abstract String getUserId();

    public abstract void setUserId(String userId);

    public abstract void setWarning(boolean state);

    public abstract boolean getWarning();

    public abstract void setWarningText(String warningText);

    public abstract String getWarningText();

}
