// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.request.IUploadFile;

import cexer.DataStorage;
import cexer.util.Logging;
import cexer.xmlrpc.ResultHandler;

public abstract class SubmitPage extends SecurePage {
    
    public abstract void setUserId(String userId);
    public abstract String getUserId();
    
    public abstract void setExerciseId(String exerciseId);
    public abstract String getExerciseId();
    
    public abstract void setPoints(String points);
    public abstract String getPoints();
    
    public abstract void setMaxPoints(String maxPoints);
    public abstract String getMaxPoints();
    
    public abstract void setSeed(String seed);
    public abstract String getSeed();
    
    public abstract void setChecksum(String checksum);
    public abstract String getChecksum();
    
    public abstract void setExerciseClass(String exerciseClass);
    public abstract String getExerciseClass();
    
    public abstract IUploadFile getFile();
    
    
    /**
     * Receives a submission.
     * @param cycle
     */
    public void formSubmit(IRequestCycle cycle) {
        DataStorage ds = DataStorage.getInstance();  // Properties are not necessarily loaded if we don't do this.
        
        // Read submission parameters
        String userId = getUserId();
        String checksum = getChecksum();
        String exerciseId = getExerciseId();
        String rawPoints = getPoints();
        String rawMaxPoints = getMaxPoints();
        String exerciseClass = getExerciseClass();  // Included in the fingerprint in the old specification.
        String seed = getSeed();                    // NOTE: Seed is read from the database at the moment.
        
        Logging.info("Receiving submission: "
                + "userId=" + userId
                + ", checksum=" + checksum
                + ", exerciseId=" + exerciseId
                + ", exerciseClass=" + exerciseClass
                + ", points=" + rawPoints
                + ", maxPoints=" + rawMaxPoints
                + ", seed=" + seed + " (ignored)"
        );
       
        // Parse parameters
        String courseId;
        int roundNumber;
        int exerciseNumber;
        int points = 0;
        int maxPoints = 0;
        
        try {
            String[] parts = exerciseId.split("\\.");
            if (parts.length != 3) {
                throw new Exception("Invalid exercise id format. " + parts.length + " parts instead of 3.");
            }
            
            courseId = parts[0];
            roundNumber = Integer.parseInt(parts[1]);
            exerciseNumber = Integer.parseInt(parts[2]);
            
            points = Integer.parseInt(rawPoints);
            maxPoints = Integer.parseInt(rawMaxPoints);
        } catch(Exception e) {
            Logging.warn(this, "Malformatted submission received. Received exerciseId: '" + exerciseId + "'. Expected format: courseId.roundNumber.exerciseNumber, where roundNumer and exerciseNumber are integers.\n" + e);
            this.showErrorMessage(cycle,"Server cannot receive the submission (invalid exercise id).");
            return;
        }
        
        // Generate output filename
        String timestamp = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        String path = System.getProperty("submissions.directory")
                + courseId + File.separator
                + roundNumber + File.separator
                + exerciseNumber + File.separator;
        String fileName = userId + "-" + timestamp;
        
        // Add result to the database
        ResultHandler handler = new ResultHandler();
        Hashtable ret = handler.addResult(userId, courseId, roundNumber, exerciseNumber, points, exerciseClass,
                maxPoints, path + fileName, checksum);
        
        if (!(Boolean)ret.get("operationOk")) {
            // Show error message
            this.showErrorMessage(cycle, (String)ret.get("message"));
            return;
        }
        
        // Save the file
        IUploadFile file = getFile();
        
        try {
            this.saveSubmission(file, path, fileName);
        } catch(Exception e) {
            Logging.error(this, e.toString());
            this.showErrorMessage(cycle, "Server failed to save the submission. Please contact staff.");
            return;
        }
        
        // Generate feedback XML
        SubmissionFeedbackPage feedbackPage = (SubmissionFeedbackPage)cycle.getPage("SubmissionFeedback");
        
        feedbackPage.setStatus("OK");
        feedbackPage.setMessage((String)ret.get("message"));
        feedbackPage.setSubmissionGrade((String)ret.get("grade"));
        //feedbackPage.setCurrentGrade((String)ret.get("bestgrade"));
        //feedbackPage.setCurrentSubmissions((int)ret.get("submissions");
        cycle.activate(feedbackPage);
    }
    
    /**
     * Saves the submitted file on the disk
     */
    private void saveSubmission(IUploadFile file, String path, String fileName) throws Exception {
        // Check for empty file
        if (file == null || (int)file.getSize() < 1) {
            Logging.info(this, "No submission file or empty file.");
            return;
        }
        
        // Check if the output folder exists. Create if necessary.
        File dir = new File(path);
        if (!dir.exists()) {
            if (!dir.mkdirs()) {
                throw new Exception("Failed to create folder " + dir);
            } 
        }
        
        // Read the data
        byte fileData[] = new byte[(int)file.getSize()];
        InputStream fileInput = file.getStream();
        fileInput.read(fileData);
        
        // Save the data
        FileOutputStream outputStream = null;
        boolean success = true;
        try {
            File outputFile = new File(dir, fileName);
            outputStream = new FileOutputStream(outputFile);
            outputStream.write(fileData);
        } catch (IOException e) {
            Logging.error(this, "Failed to save submission\n" + e);
            success = false;
        } finally {
            outputStream.close();
        }
        
        if (!success) {
            throw new Exception("Failed to save the submission");
        }
    }

    /**
     * Shows the feedback XML with the specified error message.
     */
    private void showErrorMessage(IRequestCycle cycle, String message) {
        SubmissionFeedbackPage feedbackPage = (SubmissionFeedbackPage)cycle.getPage("SubmissionFeedback");
        feedbackPage.setStatus("ERROR");
        feedbackPage.setMessage(message);
        cycle.activate(feedbackPage);
    }
}
