// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import cexer.util.Logging;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import java.io.PrintWriter;
import java.io.StringWriter;

/**
 * @author cefallen
 *
 */
public class MailerService {

    private static MailerService serviceInstance;
    
    private static boolean enabled;
        
    /**
     * Private constructor.
     */
    private MailerService() { } 
    
    /**
     * Gets the <code>MailerService</code> instance.
     * 
     * @return MailerService instance
     */
    public static synchronized MailerService getInstance() {
        if(serviceInstance == null) {
            if(System.getProperty("mailerService.enabled").toLowerCase().equals("true"))
                enabled = true;
            else enabled = false;
            serviceInstance = new MailerService();
        }
        return serviceInstance;
    }
    
    private String getStackTrace(Throwable t)
    {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw, true);
        t.printStackTrace(pw);
        pw.flush();
        sw.flush();
        return sw.toString();
    }

    /**
     * @param caller
     * @param infoMessage
     */
    public boolean mailToAdminList(Object caller, String infoMessage) {
        return mailToAdminList(caller, infoMessage, null);
    }
    
    /**
     * @param caller
     * @param e
     */
    public boolean mailToAdminList(Object caller, Throwable e) {
        return mailToAdminList(caller, null, e);
    }
    
    /**
     * @param caller
     * @param infoMessage
     * @param e
     */
    public boolean mailToAdminList(Object caller, String infoMessage, Throwable e) {        
        String address = System.getProperty("adminList.address");
        if(address == null)
            address = "";
        String[] addresses = null;
        if(address.contains(",")) {
            addresses = address.split(",");
            for(int i = 0; i<addresses.length;i++)
                addresses[i] = addresses[i].trim();
        } else
            addresses = new String[] {address};         
        
        String message = "This is an automated message from the Trakla2 system at " 
            + System.getProperty("http.server") + "\n\n";
        
        if(infoMessage != null && !infoMessage.equals(""))
            message += "Message: " + infoMessage + "\n\n";
        
        message += "Calling page or component: " + caller + "\n\n";
        
        if(e != null)
            message += "The following Exception was encountered: \n" + getStackTrace(e) + "\n\n\n";    
        
        DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
                
        message += "This message was sent at " + df.format(new Date()) +" to the address(es) " + address + ".\nThese " +
                "messages can be disabled by editing the DataStorage.properties file. Do not reply to this message!";   
        try {
            return postMail(addresses,
                "Trakla2 automated information message",
                message, System.getProperty("mailerService.fromAddress"),
                System.getProperty("smtp.host"));
        } catch (MessagingException me) {
            me.printStackTrace();
            Logging.error(this, "Error sending information e-mail: " + me.toString());
            return false;
        }
    }
    
    public boolean mailToFeedbackList(String feedback) {
        String topic = "Trakla2 feedback (from " + System.getProperty("http.server") + ")";
        DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
        String address = System.getProperty("feedbackList.address");
        if(address == null)
            address = "";
        String[] addresses = null;
        if(address.contains(",")) {
            addresses = address.split(",");
            for(int i = 0; i<addresses.length;i++)
                addresses[i] = addresses[i].trim();
        } else
            addresses = new String[] {address};         
                
        
        feedback = "This is an automated message from the Trakla2 system at " + System.getProperty("http.server") + "\n\n" + feedback
            + "This message was sent at " + df.format(new Date()) +" to the address(es) " + address + ".\nThese " 
            + "messages can be disabled by editing the DataStorage.properties file. Do not reply to this message!";   
        
        try {
            return postMail(addresses,
                topic,
                feedback, System.getProperty("mailerService.fromAddress"),
                System.getProperty("smtp.host"));
        } catch (MessagingException me) {
            me.printStackTrace();
            Logging.error(this, "Error sending information e-mail: " + me.toString());
            return false;
        }        
    }
    
    public boolean mailToUsers(String address, String message) {
        if(address == null)
            address = "";
        String[] addresses = null;
        if(address.contains(",")) {
            addresses = address.split(",");
            for(int i = 0; i<addresses.length;i++)
                addresses[i] = addresses[i].trim();
        } else
            addresses = new String[] {address};
        
        try {
            return postMail(addresses,
                "TRAKLA2 Lost password",
                message, System.getProperty("mailerService.fromAddress"),
                System.getProperty("smtp.host"));
        } catch (MessagingException me) {
            me.printStackTrace();
            Logging.error(this, "Error sending information e-mail: " + getStackTrace(me));
            return false;
        } 
        
    }
    
    /**
     * @param recipients
     * @param subject
     * @param message
     * @param from
     * @param smtpHost
     * @throws MessagingException
     */
    private boolean postMail(String recipients[ ], String subject, String message, String from, String smtpHost) throws MessagingException {
        boolean debug = false;
        if(!enabled) {
            Logging.info(this, "MailerService is not enabled, no mail was sent.");
            return false;
        }
        
        Properties props = new Properties();
        props.put("mail.smtp.host", smtpHost);

        Session session = Session.getDefaultInstance(props, null);
        session.setDebug(debug);

        // create a message
        Message msg = new MimeMessage(session);

        // set the from and to address
        InternetAddress addressFrom = new InternetAddress(from);
        msg.setFrom(addressFrom);

        InternetAddress[] addressTo = new InternetAddress[recipients.length];
        for (int i = 0; i < recipients.length; i++) {
            addressTo[i] = new InternetAddress(recipients[i]);
        }
        msg.setRecipients(Message.RecipientType.TO, addressTo);

        // Setting the Subject and Content Type
        msg.setSubject(subject);
        msg.setContent(message, "text/plain");
        Transport.send(msg);
        return true;
    }
}
